<?php declare(strict_types=1);
namespace DiviManagerPro\Action;

defined( 'ABSPATH' ) or die( 'Unexpected access' );

use DiviManager\Action\AbstractAction;
use DiviManager\Helper\Sodium;
use DiviManager\Helper;
use DiviManager\Service\Queue;
use DiviManager\Service\Site as SiteService;
use DiviManagerPro\Service\SiteServer as SiteServerService;
use DiviManagerPro\Service\SiteProvision as SiteProvisionService;
use DiviManager\Service\Profile as ProfileService;
use DiviManagerPro\Service\ProfileServer as ProfileServerService;
use DiviManagerPro\Service\ProfileIntegration as ProfileIntegrationService;
use DiviManagerPro\Integration\AbstractIntegration as Integration;
use DiviManagerPro\Service\IntegrationServer as IntegrationServerService;

/**
 * Perform site provision
 */
class SiteProvision extends AbstractAction
{
    const ACTION = 'provision';

    const STATUS_NEW  = 'NEW';
    const STATUS_DNS  = 'DNS';
    const STATUS_WP   = 'WP';
    const STATUS_SSL  = 'SSL';
    const STATUS_DONE = 'DONE';

    /**
     * @var Queue
     */
    protected $queue;

    /**
     * @var ProfileService
     */
    protected $profileService;

    /**
     * @var ProfileServerService
     */
    protected $profileServerService;

    /**
     * @var ProfileIntegrationService
     */
    protected $profileIntegrationService;

    /**
     * @var SiteService
     */
    protected $siteService;

    /**
     * @var SiteServerService
     */
    protected $siteServerService;

    /**
     * @var IntegrationServer
     */
    protected $integrationServerService;

    /**
     * @var array map current step to the next step
     */
    protected $nextSteps = [];

    public function __construct(
        Queue $queue,
        SiteProvisionService $provision,
        ProfileService $profile,
        ProfileServerService $profileServer,
        ProfileIntegrationService $profileIntegration,
        SiteService $siteService,
        SiteServerService $siteServerService,
        IntegrationServerService $integrationServerService
    )
    {
        $this->setQueue($queue);
        $this->setProvisionService($provision);
        $this->setProfileService($profile);
        $this->setProfileServerService($profileServer);
        $this->setProfileIntegrationService($profileIntegration);
        $this->setSiteService($siteService);
        $this->setSiteServerService($siteServerService);
        $this->setIntegrationServerService($integrationServerService);

        $this->nextSteps = [
            self::STATUS_NEW => self::STATUS_DNS,
            self::STATUS_DNS => self::STATUS_WP,
            self::STATUS_WP  => self::STATUS_SSL,
            self::STATUS_SSL => self::STATUS_DONE
        ];
    }

    public function setQueue(Queue $queue)
    {
        $this->queue = $queue;
        return $this;
    }

    public function setProvisionService(SiteProvisionService $provision)
    {
        $this->provisionService = $provision;
        return $this;
    }

    public function setProfileService(ProfileService $profile)
    {
        $this->profileService = $profile;
        return $this;
    }

    public function setProfileServerService(ProfileServerService $profileServer)
    {
        $this->profileServerService = $profileServer;
        return $this;
    }

    public function setProfileIntegrationService(ProfileIntegrationService $profileIntegration)
    {
        $this->profileIntegrationService = $profileIntegration;
        return $this;
    }

    public function setSiteService(SiteService $siteService)
    {
        $this->siteService = $siteService;
        return $this;
    }

    public function setSiteServerService(SiteServerService $siteServerService)
    {
        $this->siteServerService = $siteServerService;
        return $this;
    }

    public function setIntegrationServerService(IntegrationServerService $integrationServerService)
    {
        $this->integrationServerService = $integrationServerService;
        return $this;
    }

    public function registerActions($worker)
    {
        $worker->actions[self::ACTION] = [$this, 'provision'];
    }

    /**
     * Determine the next step from the curent step
     *
     * This order can depend on which integrations are used...
     * For now, hardcoded to dns first
     */
    public function nextStep($currentStep)
    {
        if (isset($this->nextSteps[$currentStep])) {
            return $this->nextSteps[$currentStep];
        }

        throw new \RuntimeException('Unknown step for provision request: ' . $currentStep);
    }

    /**
     * Perform a site provisioning step
     *
     * A provision has multiple steps, depending on the integrations enabled
     */
    public function provision($site, $args) {
        $step = $args['status'];

        if (self::STATUS_NEW == $step) {
            // this is a new provision
            // do nothing and move to the next step
        } elseif (self::STATUS_DNS == $step) {
            $this->doDns($site);
        } elseif (self::STATUS_WP == $step) {
            $this->doProvision($site);
        } elseif (self::STATUS_SSL == $step) {
            $this->doSsl($site);
        } elseif (self::STATUS_DONE == $step) {
            // should not have been called whenthe step is done, that is the finished step
            throw new \RuntimeException('Provision step called when status is done');
        } else {
            throw new \RuntimeException('Unsupported step: ' . $step);
        }

        $nextStep =  $this->nextStep($step);

        if (self::STATUS_DONE === $nextStep) {
            // provision is done, do not add another queue event
            return $site;
        }

        $this->queue->addEventForSiteToQueue(self::ACTION, $site, ['status' => $nextStep]);
    }

    /**
     * Setup dns for a site provision request
     */
    public function doDns($site)
    {
        //get the profile for the site
        $profile = $this->profileService->getProfile($site->getProfileID());
        if (null === $profile) {
            // site does not have a profile
            // nothing else to do
            // Is this an error?  No, sites can simply not have a profile assigned.
            return $site;
        }

        // get ip from server
        $server = $this->profileServerService->getServerForProfile($profile);
        if (empty($server)) {
            // no server, can not proceed
            error_log( __CLASS__.':'.__FUNCTION__ . ' Profile has no server set: . $profile->getName()');
            Helper::adminMessage( __( 'Profile has no server set.') . $profile->getName(), Helper::ERROR);
            return $site;
        }
        $server = $server[0];

        $dnsIntegration = $this->profileIntegrationService->getIntegrationForProfileByType($profile, Integration::SERVICE_DNS);

        if (null === $dnsIntegration) {
            // no dns set, skip this step
        } else {
            $domain = $this->siteService->justDomain($site->getDomain());
            $result = $dnsIntegration->registerDomain($domain, $server->getIp());
            if (isset($result->id)) {
                // id is set, successful add of domain
                error_log( __CLASS__.':'.__FUNCTION__ . ' Added domain:' . $domain);
                Helper::adminMessage( __( 'Domain added: ') . $domain, Helper::SUCCESS);
            } else {
                error_log( __CLASS__.':'.__FUNCTION__ . ' Failed to add domain:' . $domain);
                Helper::adminMessage( __( 'Failed to add domain: ') . $domain, Helper::ERROR);
            }
        }

        return $site;
    }

    /**
     * Request a site provision on the remote service
     * This will create the site on the remote server
     */
    public function doProvision($site)
    {
        $domain = $this->siteService->justDomain($site->getDomain());
        $integration = $this->getIntegrationForSite($site);
        $server = $this->getServerForSite($site);
        $installID = $integration->provisionNewSiteOnServer($domain, $this->siteService->liveUrlForSite($site), $server);
        $this->siteServerService->addSiteToServer($site, $server, $installID);
    }

    /**
     * Enable ssl on the site, if supported
     */
    public function doSsl($site)
    {
        $integration = $this->getIntegrationForSite($site);
        $installID = $integration->enableAutoSsl($site);
    }

    /**
     * Get the server/service this site is running on
     *
     * @param Site
     *
     * @return Server
     */
    public function getServerForSite($site)
    {
        //get the profile for the site
        $profile = $this->profileService->getProfile($site->getProfileID());
        if (null === $profile) {
            // site does not have a profile
            // nothing else to do
            // Is this an error?  No, sites can simply not have a profile assigned.
            return $site;
        }

        // get ip from server
        $server = $this->profileServerService->getServerForProfile($profile);
        if (empty($server)) {
            // no server, can not proceed
            error_log( __CLASS__.':'.__FUNCTION__ . ' Profile has no server set: . $profile->getName()');
            Helper::adminMessage( __( 'Profile has no server set.') . $profile->getName(), Helper::ERROR);
            return $site;
        }
        $server = $server[0];

        return $server;
    }

    /**
     * Get the integration service for the server the site is running on
     *
     * @param site
     *
     * @return Integration
     */
    public function getIntegrationForSite($site)
    {
        $server = $this->getServerForSite($site);
        $integration =  $this->integrationServerService->getIntegrationForServer($server);
        if (null === $integration || (is_array($integration) && empty($integration))) {
            throw new \RuntimeException('Could not find integration service for site');
        }

        if (is_array($integration)) {
            return $integration[0];
        }

        return $integration;
    }

    public function uhter() {
        $this->siteService = $this->registry->getService(\DiviManager\Service\Site::class);
        $this->profileService = $this->registry->getService(\DiviManager\Service\Profile::class);
        $this->queueService = $this->registry->getService(\DiviManager\Service\Queue::class);
        $this->integrationService = $this->registry->getService(\DiviManagerPro\Service\Integration::class);
        $this->profileIntegrationService = $this->registry->getService(\DiviManagerPro\Service\ProfileIntegration::class);
        $this->profileServerService = $this->registry->getService(\DiviManagerPro\Service\ProfileServer::class);
        $this->integrationServerService = $this->registry->getService(\DiviManagerPro\Service\IntegrationServer::class);
        
        $profile = $this->profileService->getProfile($site->getProfileID());

        // this needs rewritten
        $dnsFirst = false; // CHANGE BACK BEFORE PULL REQUEST!!!
        if ($dnsFirst) {
            // do dns first
            // get ip from server
            $server = $this->profileServerService->getServerForProfile($profile);
            if (null === $server) {
                // no server, can not proceed
                error_log( __CLASS__.':'.__FUNCTION__ . ' Profile has no server set: . $profile->getName()');
                Helper::adminMessage( __( 'Profile has no server set.') . $profile->getName(), Helper::ERROR);
                return site;
            }
            $server = $server[0];

            $integrations = $this->profileIntegrationService->getIntegrationsForProfile($profile);
            foreach ($integrations as $integration) {
                if ($integration::SERVICE_DNS == $integration->getType()) {
                    break;
                }
            }
            if (null === $integration) {
                // no dns set, skip this step
            } else {
                $domain = $this->siteService->justDomain($site->getDomain());
                $result = $integration->registerDomain($domain, $server->getIp());
                if (isset($result->id)) {
                    // id is set, successful add of domain
                } else {
                    error_log( __CLASS__.':'.__FUNCTION__ . ' Failed to add domain:' . $domain);
                    Helper::adminMessage( __( 'Failed to add domain') . $domain, Helper::ERROR);
                    return site;
                }
            }

            // then add site to server
            $integration = $this->integrationServerService->getIntegrationForServer($server);
            $installID = $integration->provisionNewSiteOnServer($domain, $this->siteService->liveUrlForSite($site), $server);
            $this->siteServerService->addSiteToServer($site, $servier, $installID);

        } else {
            // add site to service
            // get ip from site
            // add to dns
        }
        /*
        //if (true === $provision && true == $profile->doesProvision())
        if (true === $provision) {
            if (true === $platform->doesProvision()) {
                $this->queueService->addEventForSiteToQueue(\DiviManagerPro\Action\Platform::ACTION, $site);
                error_log( __CLASS__.':'.__FUNCTION__ . ' Added provision to queue');
                Helper::adminMessage( __( 'Site added to queue for provisioning.  Please allow a few minutes to complete.'), Helper::SUCCESS);
            } else {
                error_log( __CLASS__.':'.__FUNCTION__ . ' No provision');
                Helper::adminMessage( __( 'Profile does not support provisioning a new site on the platform.'), Helper::ERROR);
            }
        }

        //if (true === $clone && true == $profile->doesClone())
        // clone currently controls whether a site is provision on the platform at all
        if (true === $clone) {
            // send this to the queue
            $this->queueService->addEventForSiteToQueue(\DiviManagerPro\Action\Platform::ACTION_CLONE, $site);
            error_log( __CLASS__.':'.__FUNCTION__ . ' Added clone to queue');
            Helper::adminMessage( __( 'Site added to queue for cloning.  Please allow a few minutes to complete'), Helper::SUCCESS);
        }
         */
    }
}
