<?php declare(strict_types=1);
namespace DiviManagerPro\Integration\Dnsmadeeasy;

defined( 'ABSPATH' ) or die( 'Unexpected access' );

use DNSMadeEasyAPI\DNSMadeEasyAPI;
use DiviManager\Registry;
use DiviManager\Helper;
use DiviManager\Helper\Option;
use DiviManagerPro\Service\Integration as IntegrationService;
use DiviManagerPro\Integration\AbstractIntegration;

/**
 * Manage the integration with dnsmadeeasy
 */
class Dnsmadeeasy extends AbstractIntegration
{
    public const OPTION_ENABLED = 'dnsmadeeasy_enabled';

    /**
     * @var string
     *
     * The hook prefix for our actions so we can hook into events and stay synchronized with the main plugin
     */
    protected $hookPrefix;

    /**
     * @var array
     *
     * Our settings
     */
    protected $settings;

    protected $registry;
    protected $option;

    public function __construct($hookPrefix)
    {
        $this->hookPrefix = $hookPrefix;
        $this->setId('dnsmadeeasy');
        $this->setType(self::SERVICE_DNS);
        $this->setName('DNS Made Easy');

        add_action( $this->hookPrefix . '_services_init', [$this, 'services_init'], 12);

        $this->settings = [
            'enabled' => [
                'name' => 'DNS Made Easy Enabled',
                'description' => '',
                'id' => 'enabled',
                'type' => 'checkbox',
                'validation' => null,
            ],
            'apiurl' => [
                'name' => 'DME API Url',
                'description' => 'Dnsmadeeasy API URL, should be set to https://api.dnsmadeeasy.com/',
                'id' => 'apiurl',
                'type' => 'text',
                'validation' => null,
            ],
            'apikey' => [
                'name' => 'DME API Key',
                'description' => 'API key for dnsmadeeasy service',
                'id' => 'apikey',
                'type' => 'password',
                'validation' => null,
            ],
            'secretkey' => [
                'name' => 'DME Secret Key',
                'description' => 'Secret key for dnsmadeeasy service',
                'id' => 'secretkey',
                'type' => 'password',
                'validation' => null,
            ],
            'domainid' => [
                'name' => 'DME Domain ID',
                'description' => 'Domain ID on dnemadeeasy to which the staging domains will be added',
                'id' => 'domainid',
                'type' => 'password',
                'validation' => null,
            ],
            'serverip' => [
                'name' => 'DME Server IP',
                'description' => 'Default IP of the server on which the sites are being installed',
                'id' => 'serverip',
                'type' => 'text',
                'validation' => null,
            ],
            'staging' => [
                'name' => 'Staging Domain',
                'description' => 'The domain the websites will be added to as subdomains',
                'id' => 'staging',
                'type' => 'text',
                'validation' => null,
            ],
        ];
    }

    /**
     * Setup our services.
     *
     * Every plugin will be adding their own
     */
    public function services_init(Registry $registry)
    {
        $this->registry = $registry;
        $this->registry->setService(__CLASS__, $this); 
        //$this->registry->setService(Admin::class, AdminFactory::class);

        $is = $this->registry->getService(IntegrationService::class);
        $is->register($this);
		$this->option = $this->registry->getService(Option::class); 
    }

    public function getSettings()
    {
        $settings = [];
        $options = $this->option->getOption('integrations');
        if (! isset($options[$this->getId()])) {
            $options[$this->getId()] = [];
        }
        foreach ($this->settings as $key => $setting) {
            $settings[$key] = $setting;
            if (isset($options[$this->getId()][$setting['id']])) {
                $settings[$key]['value'] = $options[$this->getId()][$setting['id']];
            }
        }
		return $settings;
    }

    public function updateSettings($settings)
    {
        $options = $this->option->getOption('integrations');
        if (! isset($options[$this->getId()])) {
            $this->options[$this->getId()] = [];
        }

        foreach ($this->settings as $setting) {
            if (isset($settings[$setting['id']])) {
                $options[$this->getId()][$setting['id']] = sanitize_text_field( $settings[$setting['id']] );
            }
        }

        $this->option->setOption('integrations', $options);
    }

    /**
     * Get a configured DNSMadeEasyAPI instance
     */
    public function getDMEAPI()
    {
        $options = $this->getOptions();

        if (empty($options)) {
            throw new \Exception('DNSMadeEaasy has not been configured yet, unable to proceed.');
        }
        $dmeapi = new DNSMadeEasyAPI($options['apiurl'], $options['apikey'], $options['secretkey']);
        return $dmeapi;
    }

    /**
     * Add domain to dnsmadeeasy
     */
    public function registerDomain($domain, $ip)
    {
        $options = $this->getOptions();

        // add to dme
        $dmeapi = $this->getDMEAPI();
        return $dmeapi->addRecord($domain, $options['domainid'], $ip);//$options['serverip']);
    }
}
