<?php declare(strict_types=1);
namespace DiviManagerPro\Integration\Serverpilot;

defined( 'ABSPATH' ) or die( 'Unexpected access' );

use DiviManager\Registry;
use DiviManager\Helper;
use DiviManager\Helper\Option;
use DiviManagerPro\Service\Integration as IntegrationService;
use DiviManagerPro\Service\Server as ServerService;
use DiviManagerPro\Service\IntegrationServer as IntegrationServerService;
use DiviManagerPro\Integration\AbstractIntegration;
use ServerPilotAPI\ServerPilotAPI;

/**
 * Manage the integration with serverpilot
 */
class Serverpilot extends AbstractIntegration
{
    public const OPTION_ENABLED = 'serverpilot_enabled';

    /**
     * @var string
     *
     * The hook prefix for our actions so we can hook into events and stay synchronized with the main plugin
     */
    protected $hookPrefix;

    /**
     * @var array
     *
     * Our settings
     */
    protected $settings;

    /**
     * @var IntegrationServerService
     */
    protected $integrationServerService;

    /**
     * @var ServerService
     */
    protected $serverService;

    protected $options;
    protected $option;
    protected $registry;

    public function __construct($hookPrefix)
    {
        $this->hookPrefix = $hookPrefix;
        $this->setId('serverpilot');
        $this->setType(self::SERVICE_HOST);
        $this->setName('ServerPilot');
        $this->options = [];

        add_action( $this->hookPrefix . '_services_init', [$this, 'services_init'], 12);

        $this->settings = [
            'enabled' => [
                'name' => 'ServerPilot Enabled',
                'description' => '',
                'id' => 'enabled',
                'type' => 'checkbox',
                'validation' => null,
            ],
            'apiurl' => [
                'name' => 'API Url',
                'description' => 'ServerPilot API URL, should be set to https://api.serverpilot.io',
                'id' => 'apiurl',
                'type' => 'text',
                'validation' => null,
            ],
            'clientid' => [
                'name' => 'Client ID',
                'description' => 'Client ID for the ServerPilot account',
                'id' => 'clientid',
                'type' => 'password',
                'validation' => null,
            ],
            'apikey' => [
                'name' => 'API Key',
                'description' => 'API Key for the client ID corredsponding to the ServerPilot account',
                'id' => 'apikey',
                'type' => 'password',
                'validation' => null,
            ],
            'sshkey' => [
                'name' => 'SSH Key',
                'description' => 'SSH Key to associate with the new site, if you have one',
                'id' => 'sshkey',
                'type' => 'text',
                'validation' => null,
            ],
            'adminuser' => [
                'name' => 'Default Admin User',
                'description' => 'Default admin user for new sites',
                'id' => 'adminuser',
                'type' => 'text',
                'validation' => null,
            ],
            'adminemail' => [
                'name' => 'Default Admin Email',
                'description' => 'Default admin email for new sites',
                'id' => 'adminemail',
                'type' => 'text',
                'validation' => null,
            ],
            'phpruntime' => [
                'name' => 'PHP Runtime',
                'description' => 'Default php runtime for new sites',
                'id' => 'phpruntime',
                'type' => 'text',
                'validation' => null,
            ],
        ];
    }

    /**
     * Setup our services.
     */
    public function services_init(Registry $registry)
    {
        $this->registry = $registry;
        $is = $this->registry->getService(IntegrationService::class);
        $is->register($this);
        $this->option = $this->registry->getService(Option::class); 
        $this->integrationServerService = $this->registry->getService(IntegrationServerService::class);
        $this->serverService = $this->registry->getService(ServerService::class);
    }

    public function getSettings()
    {
        $settings = [];
        $options = $this->option->getOption('integrations');
        if (! isset($options[$this->getId()])) {
            $options[$this->getId()] = [];
        }
        foreach ($this->settings as $key => $setting) {
            $settings[$key] = $setting;
            if (isset($options[$this->getId()][$setting['id']])) {
                $settings[$key]['value'] = $options[$this->getId()][$setting['id']];
            }
        }
		return $settings;
    }

    public function updateSettings($settings)
    {
        $options = $this->option->getOption('integrations');
        if (! isset($options[$this->getId()])) {
            $options[$this->getId()] = [];
        }

        foreach ($this->settings as $setting) {
            if (isset($settings[$setting['id']])) {
                $options[$this->getId()][$setting['id']] = sanitize_text_field( $settings[$setting['id']] );
            }
        }

        $this->option->setOption('integrations', $options);
    }

    public function getSPAPI()
    {
        $options = $this->getOptions();
        $spapi = new ServerPilotAPI($options['apiurl'], $options['clientid'], $options['apikey']);
        $spapi->setAdminUser($options['adminuser']);
        $spapi->setAdminEmail($options['adminemail']);
        return $spapi;
    }

    /**
     * List the sites hosted on the service
     */
    public function installs()
    {
        $spapi = $this->getSPAPI();
        $sites = $spapi->sites();
        $spapi->close();

        return $sites;
    }

    /**
     * List the servers registered with the service
     */
    public function servers()
    {
        $spapi = $this->getSPAPI();
        $sites = $spapi->servers();
        $spapi->close();

        return $sites;
    }

    /**
     * Get the registered domains on dme
     */
    public function domains()
    {
        $options = $this->getOptions();
        $dmeapi = $this->getDMEAPI();
        return $dmeapi->getRecords($options['dmedomainid']);
    }

    /**
     * Get filesizes for site
     * return wp-content and db usage, maybe more
     */
    public function filesizes($site)
    {
        $paths = $this->paths($site->getInstallID());
        var_dump($paths);
        //$filesize = `du -s {$paths[]}`;
        return [];
    }

    /**
     * Get details for a site
     */
    public function getSite($site)
    {
        $installID = $site->getInstallID();
        if (null === $installID) {
            return [];
        }
        $spapi = $this->getSPAPI();
        $site = $spapi->site($installID);
        $spapi->close();

        return $site;
    }

    /**
     * Provision a new site on the service
     *
     * $name is the sld, the shepherdsloft part of shepherdsloft.com
     * $name is used for the subdomain, the site name, and the site title
     */
    public function provisionNewSiteOnServer($name, $domain, $server)
    {
        error_log(
            __CLASS__.__FUNCTION__.
            ': Beginning install for site: ' . $name
        );

        $options = $this->getOptions();
        $spapi = $this->getSPAPI();

        $length = strlen($name);
        if (32 < $length) {
            $length = 32;
        }
        $truncatedUserName = substr($name, 0, $length);
        $user = $spapi->createUser($server->getServerId(), $truncatedUserName, bin2hex(\random_bytes(10)), $options['sshkey']);
        $actionid = $user->actionid;
        while(true === $spapi->actionOpen($actionid)) {
            error_log(
                __CLASS__.__FUNCTION__.
                ': Waiting on createUser action to complete'
            );
            $this->doNothing();
        }
        if (false === $spapi->actionSuccess($actionid)) {
            throw new \RuntimeException('Error creating user for new install');
        }
        //$domain = $name . '.' . $options['dmestaging'];
        $site = $spapi->createSite($name, $user->data->id, $domain, $options['phpruntime']);
        $actionid = $site->actionid;
        while(true === $spapi->actionOpen($actionid)) {
            $this->doNothing();
        }
        if (false === $spapi->actionSuccess($actionid)) {
            throw new \RuntimeException('Error creating site for new install');
        }

        error_log(
            __CLASS__.__FUNCTION__.
            ': Finished install for site: ' . $name
        );

        return $site->data->id;
    }

    /**
     * Migrate one site to another
     */
    public function migrateSite($sourceID, $siteID)
    {
        $spapi = $this->getSPAPI();

        $src = $spapi->paths($sourceID);
        $dest = $spapi->paths($siteID);

        list($fromUser, $fromApp) = $src;
        list($toUser, $toApp) = $dest;
        $siteAction = new SiteAction;
        return $siteAction->migrate($fromUser, $fromApp, $toUser, $toApp);
    }

    /**
     * Clone an existing site on the service
     */
    public function cloneInstall($sourceID, $name)
    {
        $spapi = $this->getSPAPI();

        try {
            // create the new site, which also adds to DME
            $siteID = $this->newInstall($name);
            // migrate source to new site
            //$this->migrateSite($sourceID, $siteID);
        } catch (\Exception $e) {
            throw new \RuntimeException('Failed to clone to new site: '. $e->getMessage());
        }

        return $siteID;
    }

    /**
     * Get path information for site
     */
    public function paths ($siteID)
    {
        $spapi = $this->getSPAPI();
        return $spapi->paths($siteID);
    }

    /**
     * Deactivate a site
     */
    public function deactivate($id)
    {
    }

    /**
     * Change setting of a site
     */
    public function alter($id, $setting, $value)
    {
    }

    public function doNothing()
    {
        sleep(1);
    }

    /**
     * Enable auto-ssl on site
     */
    public function enableAutoSsl($site)
    {
        $spapi = $this->getSPAPI();
        $response = $spapi->autosslSite($site->getInstallID());
        $actionid = $response->actionid;
        while(true === $spapi->actionOpen($actionid)) {
            $this->doNothing();
        }
        if (false === $spapi->actionSuccess($actionid)) {
            throw new \RuntimeException('Error doing autossl for site');
        }
        $spapi->close();
    }

    /**
     * Enable auto-ssl and force-ssl on site
     */
    public function enableForceSsl($site)
    {
        $spapi = $this->getSPAPI();
        $response = $spapi->forcesslSite($site->getInstallID());
        $actionid = $response->actionid;
        while(true === $spapi->actionOpen($actionid)) {
            $this->doNothing();
        }
        if (false === $spapi->actionSuccess($actionid)) {
            throw new \RuntimeException('Error doing autossl for site');
        }
        $spapi->close();
    }

    /**
     * Add a domain to the site
     */
    public function addDomainToSite($domain, $site)
    {
        $spapi = $this->getSPAPI();
        $response = $spapi->addDomainToSite($site->getInstallID(), $domain);
        $actionid = $response->actionid;
        while(true === $spapi->actionOpen($actionid)) {
            $this->doNothing();
        }
        if (false === $spapi->actionSuccess($actionid)) {
            throw new \RuntimeException('Error doing addDomain for site');
        }
        $spapi->close();
    }

    /**
     * Import the servers from the service
     */
    public function importServers()
    {
        $servers = $this->servers();
        $knownServers = $this->integrationServerService->getServersForIntegration($this);

        $found = false;
        foreach ($servers as $importServer) {
            foreach ($knownServers as $knownServer) {
                if ($importServer->id == $knownServer->getServerID()) {
                    $found = true;
                    Helper::adminMessage('Found existing server and updating: ' . $importServer->id, Helper::SUCCESS);
                    // found an existing server
                    // update its data with the latest
                    $knownServer->setIP($importServer->lastaddress);
                    $knownServer->setName($importServer->name);
                    $this->serverService->updateServer($knownServer);
                    break;
                }
            }

            if (false === $found) {
                Helper::adminMessage('Importing new server: ' . $importServer->id, Helper::SUCCESS);
                $newServer = $this->serverService->newModel();
                $newServer->setServerID($importServer->id);
                $newServer->setName($importServer->name);
                $newServer->setIP($importServer->lastaddress);
                $newServer = $this->serverService->createServer($newServer);
                $this->integrationServerService->addServerToIntegration($newServer, $this);
            }

            $found = false;
        }
    }
}
