<?php declare(strict_types=1);
namespace DiviManagerPro\Page;

defined( 'ABSPATH' ) or die( 'Unexpected access' );

use DiviManager\Helper;
use DiviManagerPro\Service\Integration as IntegrationService;

/**
 * Configure and manage service integrations
 */
class Integration
{
    protected $formHelper;
    protected $integrationService;

    /**
     * @var String path of our main plugin file
     */
    protected $mainPluginFile;

    public function __construct(
        $formHelper,
        IntegrationService $integrationService,
        $mainPluginFile
    ) {
        $this->formHelper = $formHelper;
        $this->setIntegrationService($integrationService);
        add_action( 'admin_enqueue_scripts', [$this, 'enqueue_scripts'] );
        $this->mainPluginFile = $mainPluginFile;
    }

    public function setIntegrationService(IntegrationService $s)
    {
        $this->integrationService = $s;
    }

    /**
     * Enqueue javascript for pages
     *
     * https://developer.wordpress.org/plugins/javascript/enqueuing/
     */
    public function enqueue_scripts($hook)
    {
        if ('manager_page_divimanager-integration' == $hook) {
            if( ! \wp_script_is( 'integrationForm.js', 'registered' ) ){
                $this->register_scripts($hook);
            }

            \wp_enqueue_script( 'form.js' );
            \wp_enqueue_script( 'integrationForm.js' );
        }
    }

    /**
     * Register our scripts so wordpress knows about them
     */
    public function register_scripts($hook)
    {
        \wp_register_script(
            'integrationForm.js', 
            \plugins_url( '/js/integrationForm.js', $this->mainPluginFile ),
            [ 'jquery' ],
            false,
            false,
        );

        \wp_localize_script(
            'integrationForm.js',
            'DMPROintegration', [
                'strings' => [
                    'saved' => __( 'Settings Saved', 'text-domain' ),
                    'error' => __( 'Error', 'text-domain' )
                ],
                'api'     => [
                    'url'   => esc_url_raw( rest_url( 'dmpro/v1/integration/settings' ) ),
                    'nonce' => wp_create_nonce( 'wp_rest' )
                ]
            ]
        );
    }

    /**
     * Main page
     */
    public function index ()
    {
        if (! empty($_POST)) {
            $integrationID = isset($_POST['integration']) ? sanitize_text_field($_POST['integration']) : null;
        } else {
            $integrationID = null;
        }
?>
    <div class="wrap">
        <h1 class="wp-heading-inline"><?php echo esc_html( get_admin_page_title() ); ?></h1>
        <p>DiviManager can integrate with several external services.  Each service must be configured.  Please select a service from the list below.</p>
        <form class="form" method="POST" name="integrationForm" id="integrationForm" action="">
<?php
            echo $this->makeIntegrationSelect($integrationID);
?>
            <div id="feedback">
            </div>
            <table id="integration-settings" class="form-table">
            </table>
            <?php submit_button( __( 'Save', 'text-domain' ) ); ?>
        </form>
    </div>
<?php
    }

    public function makeIntegrationSelect($integrationID = null)
    {
        $integrations = $this->integrationService->integrations();
        if (0 === count($integrations)) {
            return 'There are no active integrations.  Please install and configure.';
        }

        $integrationOptions = array_map(function($a) {
                return [
                    'name' => $a->getName(),
                    'value' => $a->getId(),
                ];
            },
            $integrations
        );

        $html = $this->formHelper->makeSelect('integration', 'integration', $integrationOptions, $integrationID);

        return $html;
    }
}
