<?php declare(strict_types=1);
namespace DiviManagerPro\Page;

defined( 'ABSPATH' ) or die( 'Unexpected access' );

use DiviManager\Page\Profile as ProfilePage;
use DiviManager\Service\Profile as ProfileService;
use DiviManager\Service\Theme as ThemeService;
use DiviManager\Service\Plugin as PluginService;
use DiviManager\Service\Site as SiteService;
use DiviManager\Service\ProfilePlugin as ProfilePluginService;
use DiviManager\Helper;

/**
 * Manage profiles
 */
class Profile extends ProfilePage
{
    protected $mainPluginFile;
    protected $themeService;
    protected $siteService;
    protected $pluginService;
    protected $profilePluginService;

    public function __construct(
            ProfileService $profileService,
            ThemeService $themeService,
            SiteService $siteService,
            PluginService $pluginService,
            ProfilePluginService $profilePluginService,
            $mainPluginFile
    ) {
        $this->setProfileService($profileService);
        $this->setThemeService($themeService);
        $this->setSiteService($siteService);
        $this->setPluginService($pluginService);
        $this->setProfilePluginService($profilePluginService);
        $this->mainPluginFile = $mainPluginFile;

        add_action( 'admin_enqueue_scripts', [$this, 'enqueue_scripts'] );
    }

    /**
     * Enqueue javascript for pages
     *
     * https://developer.wordpress.org/plugins/javascript/enqueuing/
     */
    public function enqueue_scripts($hook)
    {
        if ('manager_page_divimanager-profile' === $hook ||
            'admin_page_divimanager-profile-update' === $hook) {
            if( ! \wp_script_is( 'profileForm.js', 'registered' ) ){
                $this->register_scripts($hook);
            }

            \wp_enqueue_script( 'form.js' );
            \wp_enqueue_script( 'profileForm.js' );
        }
    }

    /**
     * Register our scripts so wordpress knows about them
     */
    public function register_scripts($hook)
    {
        \wp_register_script(
            'profileForm.js', 
            \plugins_url( '/js/profileForm.js', $this->mainPluginFile ),
            [ 'jquery' ],
            false,
            false,
        );

        \wp_localize_script(
            'profileForm.js',
            'DMPROprofile', [
                'strings' => [
                    'saved' => __( 'Settings Saved', 'text-domain' ),
                    'error' => __( 'Error', 'text-domain' )
                ],
                'form'     => [
                    'url'   => esc_url_raw( rest_url( 'dmpro/v1/form/profile' ) ),
                    'nonce' => wp_create_nonce( 'wp_rest' )
                ],
                'api'     => [
                    'url'   => esc_url_raw( rest_url( 'dmpro/v1/profile' ) ),
                    'nonce' => wp_create_nonce( 'wp_rest' )
                ]
            ]
        );
    }

    /**
     * Main page to manage profiles
     */
    public function manage ()
    {
        $addSiteLink =  ' <a href="' . esc_url( admin_url( 'admin.php?page=divimanager-profile-create' ) ) . '" class="page-title-action">Add New</a>';
?>
    <div class="wrap">
        <h1 class="wp-heading-inline"><?php echo esc_html( get_admin_page_title() ); ?></h1>
<?php echo $addSiteLink; ?>
<?php
        $profiles = $this->profileService->getProfiles();
        //$count = count($profiles);
        //<p>There are $count profiles currently registered with the manager.</p>

        $profiles = [];
        echo <<<EOHTML
        <table id="profiles" class="form-table">
        <thead>
            <tr><th></th><th>Profile</th><th>Description</th></tr>
        </thead>
        <tbody>
EOHTML;
        $row =<<<EOHTML
            <tr><td>%s</td><td>%s</td></tr>
EOHTML;
        $viewLink = '<a href="%s">%s</a>';
        foreach ($profiles as $profile) {
            $viewUrl = admin_url( 'admin.php?page=divimanager-profile-view&profile_id=' . $profile->getId() );
            $viewLinkUrl = sprintf($viewLink, $viewUrl, $profile->getName());
            echo sprintf($row, $viewLinkUrl, $profile->getDescription());
        }
?>
        </tbody>
        </table>
    </div>
<?php
    }

    /**
     * View details of a profile
     */
    public function view ()
    {
        if (! isset($_GET['profile_id']) || empty($_GET['profile_id'])) {
            echo '<p>Profile not specified.  Please return to the profile list and try selecting a profile.</p>';
            return;
        }

        $profileID = $_GET['profile_id'];
        $profile = $this->profileService->getProfile($profileID);

        if (null === $profile) {
            echo '<p>Profile not found.</p>';
            return;
        }

        $editUrl = admin_url( 'admin.php?page=divimanager-profile-update&profile_id=' . $profileID );

        $theme = $this->themeService->getTheme($profile->getThemeID());
        if (null === $theme) {
            $themeName = 'N/A';
        } else {
            $themeUrl = $this->themeService->getUrlForTheme($theme);
            $themeName = sprintf('<a href="%s">%s</a>', $themeUrl, $theme->getName());
        }

        $pluginBlock = '';

        $plugins = $this->profilePluginService->getPluginsForProfile($profile);
        $pluginRow = '<li><a href="%s">%s</a></li>';
        if (null === $plugins || empty($plugins)) {
            $pluginBlock = '<li>No plugins</li>';
        } else {
            foreach ($plugins as $plugin) {
                if (null === $plugin) {
                    $pluginBlock .= '<li>Plugin not found</li>';
                } else {
                    $pluginUrl = $this->pluginService->getUrlForPlugin($plugin);

                    $pluginBlock .= sprintf($pluginRow, $pluginUrl, $plugin->getName());
                }
            }
        }

        $sitesBlock = '';
        $sites = $this->siteService->getSitesByProfile($profileID);
        if (null === $sites || empty($sites)) {
            $sitesBlock = '<li>No sites</li>';
        } else {
            $siteRow = '<li><a href="%s">%s</a></li>';
            foreach ($sites as $site) {
                $siteUrl = $this->siteService->liveUrlForSite($site);
                $sitesBlock .= sprintf($siteRow, $siteUrl, $site->getName());
            }
        }

        $template =<<<EOHTML
<style>
</style>
<div class="site-detail">
    <h1>Profile: {$profile->getName()}</h1>
    <div class="left graybox" style="width: 48%;">
        <table>
            <tr><th><a href="$editUrl">Edit</a></th></tr>
            <tr><th>Description</th><td>{$profile->getDescription()}</td></tr>
            <tr><th>Theme</th><td>$themeName</td></tr>
            <tr><th>Plugins</th><td><ul>$pluginBlock</ul></td></tr>
            <tr><th>Sites</th><td><ul>$sitesBlock</ul></td></tr>
        </table>
    </div>
</div>
EOHTML;
        echo $template;
    }

    /**
     * Create a new profile
     */
    public function create ()
    {
        // is this access check necessary?
        if ( !current_user_can( 'manage_options' ) )  {
            wp_die( __( 'You do not have sufficient permissions to access this page.' ) );
        }
?>
    <div class="wrap">
        <h1 class="wp-heading-inline"><?php echo esc_html( get_admin_page_title() ); ?></h1>
        <div id="post-body" class="metabox-holder columns-3">
            <div id="post-body-content">
                <div class="meta-box-sortables ui-sortable">
                    <form class="form" method="POST" name="profileForm" id="profileForm" action="">
                        <div id="feedback">
                        </div>
                        <table id="profile-form" class="form-table">
                        </table>
                        <?php submit_button( __( 'Save Profile', 'text-domain' ) ); ?>
                    </form>
                </div>
            </div>
        </div>
	</div>
<?php
    }

    /**
     * Edit a profile
     */
    public function update ()
    {
        // is this access check necessary?
        if ( !current_user_can( 'manage_options' ) )  {
            wp_die( __( 'You do not have sufficient permissions to access this page.' ) );
        }
?>
    <div class="wrap">
        <h1 class="wp-heading-inline"><?php echo esc_html( get_admin_page_title() ); ?></h1>
        <div id="post-body" class="metabox-holder columns-3">
            <div id="post-body-content">
                <div class="meta-box-sortables ui-sortable">
                    <form class="form" method="POST" name="profileForm" id="profileForm" action="">
                        <div id="feedback">
                        </div>
                        <table id="profile-form" class="form-table">
                        </table>
                        <?php submit_button( __( 'Save Profile', 'text-domain' ) ); ?>
                    </form>
                </div>
            </div>
        </div>
	</div>
<?php
    }

    public function makeThemeSelect($themeID = null)
    {
        $html =<<<EOHTML
<select name="theme" id="theme">
    <option value=""></option>
EOHTML;

        $option =<<<EOHTML
    <option value="%s"%s>%s</option>
EOHTML;
        foreach ($this->themeService->getThemes() as $theme) {
            $selected = '';
            if (null !== $themeID && $themeID == $theme->getId()) {
                $selected = ' selected';
            }
            $html .= sprintf($option, $theme->getId(), $selected, $theme->getName());
        }

        $html .=<<<EOHTML
</select>
EOHTML;
        return $html;
    }

    public function makePluginCheckboxes($plugins)
    {
        $html =<<<EOHTML
<div>
EOHTML;

        $checkbox =<<<EOHTML
    <input type="checkbox" name="plugins[]" value="%s"%s>%s</input><br />
EOHTML;
        foreach ($this->pluginService->getPlugins() as $plugin) {
            $selected = '';
            if (null !== $plugins) {
		foreach ($plugins as $p) {
			if ($p->getId() == $plugin->getId()) {
				$selected = ' checked=true ';
				break;
			}
		}
            }
            $html .= sprintf($checkbox, $plugin->getId(), $selected, $plugin->getName());
        }

        $html .=<<<EOHTML
</div>
EOHTML;
        return $html;
    }
}
