<?php declare(strict_types=1);
namespace DiviManagerPro\Page;

defined( 'ABSPATH' ) or die( 'Unexpected access' );

use DiviManager\Helper;
use DiviManagerPro\Service\Server as ServerService;
use DiviManagerPro\Service\Integration as IntegrationService;
use DiviManagerPro\Service\IntegrationServer as IntegrationServerService;
use DiviManagerPro\Integration\AbstractIntegration as Integration;

/**
 * Display server information
 */
class Server
{
    protected $serverService;
    protected $integrationService;
    protected $integrationServerService;
    protected $formHelper;
    protected $mainPluginFile;

    public function __construct($serverService, $integrationService, $integrationServerService, $formHelper, $mainPluginFile)
    {
        $this->setServerService($serverService);
        $this->setIntegrationService($integrationService);
        $this->setIntegrationServerService($integrationServerService);
        $this->formHelper = $formHelper;
        add_action( 'admin_enqueue_scripts', [$this, 'enqueue_scripts'] );
        $this->mainPluginFile = $mainPluginFile;
    }

    public function setServerService(ServerService $s)
    {
        $this->serverService = $s;
    }

    public function setIntegrationService(IntegrationService $p)
    {
        $this->integrationService = $p;
    }

    public function setIntegrationServerService(IntegrationServerService $p)
    {
        $this->integrationServerService = $p;
    }

    /**
     * Enqueue javascript for pages
     *
     * https://developer.wordpress.org/plugins/javascript/enqueuing/
     */
    public function enqueue_scripts($hook)
    {
        if ('manager_page_divimanager-server' === $hook) {
            if( ! \wp_script_is( 'server.js', 'registered' ) ){
                $this->register_scripts($hook);
            }

            \wp_enqueue_script( 'form.js' );
            \wp_enqueue_script( 'server.js' );
        }
    }

    /**
     * Register our scripts so wordpress knows about them
     */
    public function register_scripts($hook)
    {
        \wp_register_script(
            'server.js',
            \plugins_url( '/js/server.js', $this->mainPluginFile ),
            [ 'jquery' ],
            '1.0.0',
            [
               'in_footer' => true,
            ]
        );

        \wp_localize_script(
            'server.js',
            'DMPROserver', [
                'strings' => [
                    'saved' => __( 'Servers Saved', 'text-domain' ),
                    'error' => __( 'Error', 'text-domain' )
                ],
                'api'     => [
                    'url'   => esc_url_raw( rest_url( 'dmpro/v1' ) ),
                    'nonce' => wp_create_nonce( 'wp_rest' )
                ]
            ]
        );
    }

    /**
     * Main page
     */
    public function index ()
    {
?>
    <div class="wrap">
        <h1 class="wp-heading-inline"><?php echo esc_html( get_admin_page_title() ); ?></h1>
<?php
        $this->servers();
?>
    </div>
<?php
    }

    /**
     * Show servers
     */
    public function servers ()
    {
        if (! empty($_POST)) {
            $hostID = isset($_POST['host']) ? sanitize_text_field($_POST['host']) : null;

            if (isset($_POST['submit']) && 'Import' == $_POST['submit']) {
                $integration = $this->integrationService->integration($hostID);
                if (null == $integration) {
                    Helper::adminMessage('Invalid host selected, can not import servers from host', Helper::ERROR);
                } else {
                    $integration->importServers();
                }
            }
        } else {
            $hostID = null;
        }

        echo <<<EOHTML
    <div class="wrap">
        <h1 class="wp-heading-inline"><?php echo esc_html( get_admin_page_title() ); ?></h1>
        <p>DiviManager tracks the servers available from each hosting service, using them when creating new sites.  Please select a service from the list below.  This is managing the servers for the host which DiviManager needs to know about.  DiviManager does not affect the actual servers on the hosting service.  Click "Import" to load all the servers from the hosting service.</p>
EOHTML;
?>
        <form class="form" method="POST" name="hostForm" id="hostForm" action="">
<?php
            echo $this->makeHostSelect($hostID);
            submit_button( __( 'Import', 'text-domain' ) );
?>
        </form>
        <div id="feedback">
        </div>
        <p>Following servers are registered for this hosting service.</p>
        <form id="server">
            <table id="servers" class="form-table">
                <thead>
                <tr>
                    <th></th>
                    <th>Name</th>
                    <th>IP</th>
                    <th>IPv6</th>
                    <th>Server ID</th>
                </tr>
                </thead>
                <tbody>
                <tr>
                    <td>
                        <input type="submit" value="Add" />
                    </td>
                    <td><input name="name" type="text" /></td>
                    <td><input name="ip" type="text" /></td>
                    <td><input name="ipv6" type="text" /></td>
                </tr>
                </tbody>
            </table>
        </form>
    </div>
<?php
    }

    public function makeHostSelect($hostID = null)
    {
        $integrations = $this->integrationService->integrationsWithType(Integration::SERVICE_HOST);
        if (0 === count($integrations)) {
            return 'There are no active integrations.  Please install and configure.';
        }

        $integrationOptions = array_map(function($a) {
                return [
                    'name' => $a->getName(),
                    'value' => $a->getId(),
                ];
            },
            $integrations
        );

        $html = $this->formHelper->makeSelect('host', 'host', $integrationOptions, $hostID);

        return $html;
    }
}
