<?php declare(strict_types=1);
namespace DiviManagerPro\Reporter;

defined( 'ABSPATH' ) or die( 'Unexpected access' );

use DiviManager\Registry;
use DiviManager\Service\Worker;
use DiviManager\Helper\Option;
use DiviManagerPro\DiviManagerPro;
use DiviManagerPro\Reporter\Command\Report as ReportCommand;
use DiviManagerPro\Reporter\Command\ReportFactory as ReportCommandFactory;

class Reporter
{
    public const SC_PREFIX = 'dm';
    public const CRON_DAILY_HOOK = 'divimanager_pro_reporter_daily';

    protected $hookPrefix = 'divimanager'; // need our own prefix?  dmpro?

    /**
     * var targetMonth
     */
    protected $targetMonth = null;

	/**
     * var string
     *  
     * Our main plugin file
     */     
    protected $mainPluginFile;

    public function __construct($hookPrefix)
    {
        $this->hookPrefix = $hookPrefix;
        // set a default target month
        $targetMonth = new \Datetime;
        $targetMonth->modify('last day of last month');
        $this->setTargetMonth($targetMonth);

        add_action( 'plugins_loaded', [$this, 'plugins_loaded'] );
        add_action( 'init', [$this, 'wp_init'], 12);
        add_action( $this->hookPrefix . '_admin_init', [$this, 'admin_init'], 12);
        add_action( $this->hookPrefix . '_activation', [$this, 'activation'], 12);
        add_action( $this->hookPrefix . '_deactivation', [$this, 'deactivation'], 12);
        add_action( $this->hookPrefix . '_uninstall', [static::class, 'uninstall'], 12);
        add_action( $this->hookPrefix . '_services_init', [$this, 'services_init'], 12);
    }

    /**
     * Run when the plugin is activated
     *
     * This is usually run after initial installation
     * Handle initial setup
     *
     * These are tasks which need to happen once, when the plugin is first activated
     */
    public function activation()
    {
        $this->divimanager = \DiviManager\DiviManager::getInstance(); // get the main plugin's instance for use
        $this->registry = $this->divimanager->getRegistry();
        $this->registry->setService(Admin::class, AdminFactory::class);
        $this->install();
    }

    /**
     * Runs when the plugin is deactivated
     *
     * Removes scheduled events
     */
    public function deactivation()
    {
        $timestamp = wp_next_scheduled( self::CRON_DAILY_HOOK );
        wp_unschedule_event( $timestamp, self::CRON_DAILY_HOOK );
    }

    /**
     * Runs when the plugin has been first deactivated and then deleted
     *
     * Removes all data related to the plugin
     */
    public static function uninstall ()
    {
    }

    /**
     * Runs when the plugins_loaded action is called
     * Use this to do db updates
     *
     * Run actions which need to occur every page request, but before wordpress init
     */
    public function plugins_loaded()
    {
        $this->divimanager = \DiviManager\DiviManager::getInstance(); // get the main plugin's instance for use
		$this->registry = $this->divimanager->getRegistry();
    }

    /**
     * Runs when the init action is called, after plugins are loaded
     */
    public function wp_init() {
        add_filter( 'query_vars', [$this, 'addQueryVars'] );

        // add wp-cli command
        if ( defined( 'WP_CLI' ) && WP_CLI ) {
            if ($this->isReporterEnabled()) {
                $this->registry->setService(ReportCommand::class, ReportCommandFactory::class);
                $reportCommand = $this->registry->getService(ReportCommand::class);
                \WP_CLI::add_command( 'manager report', [$reportCommand, 'report'] );
            }
        }

        if (is_admin() || (defined('WP_CLI') && WP_CLI)) {
        }

        add_action( self::CRON_DAILY_HOOK, [$this->registry->getService(Cron::class), 'cronDailyReport'] );
        if ( ! wp_next_scheduled( self::CRON_DAILY_HOOK ) ) {
            $ok = wp_schedule_event( time(), 'daily', self::CRON_DAILY_HOOK );
        }
    }

    /**
     * Runs when the admin_init action is called, admin pages are being viewed
     *
     * No need to use an is_admin() check as this is only run before loading admin pages
     */
    public function admin_init()
    {
        $this->registry->getService(Admin::class)->init();
        $this->registry->getService(Table\Site::class)->init();
        $this->registry->getService(Service\Reporter::class)->init();
		$this->option = $this->registry->getService(Option::class); 
    }

    /**
     * Setup our services.
     *
     * Every plugin will be adding their own
     */
    public function services_init(Registry $registry)
    {
		$registry->setService(__CLASS__, $this); 
        $registry->setService(Admin::class, AdminFactory::class);
        $registry->setService(Page\Main::class, Page\MainFactory::class);
        $registry->setService(Page\Report::class, Page\ReportFactory::class);
        $registry->setService(Table\Site::class, Table\SiteFactory::class);
        $registry->setService(Action\Reporter::class, Action\ReporterFactory::class);
        $registry->setService(Service\Email::class, Service\EmailFactory::class);
        $registry->setService(Service\Reporter::class, Service\ReporterFactory::class);
        $registry->setService(Cron::class, CronFactory::class);

        // add our shortcodes
        $registry->setService(Shortcode\Bloom::class, Shortcode\BloomFactory::class);
        $registry->setService(Shortcode\GfReport::class, Shortcode\GfReportFactory::class);
        $registry->setService(Shortcode\WcReport::class, Shortcode\WcReportFactory::class);
        $registry->setService(Shortcode\Report::class, Shortcode\ReportFactory::class);
        $registry->setService(Shortcode\WpJetpack::class, Shortcode\WpJetpackFactory::class);
        $registry->setService(Shortcode\WcReport::class, Shortcode\WcReportFactory::class);
        $registry->setService(Shortcode\Monarch::class, Shortcode\MonarchFactory::class);
        $registry->setService(Shortcode\MonsterInsight::class, Shortcode\MonsterInsightFactory::class);
        $registry->setService(Shortcode\WpCore::class, Shortcode\WpCoreFactory::class);
        $registry->setService(Shortcode\PhpCore::class, Shortcode\PhpCoreFactory::class);
        $registry->setService(Shortcode\Theme::class, Shortcode\ThemeFactory::class);

        // add our action to the worker service
        $worker = $registry->getService(Worker::class);
        $worker->addAction($registry->getService(Action\Reporter::class));
	}

    /**
     * install db schema
     *
     * detects the version of the schema
     * if different, runs the script
     */
    protected function install ()
    {
        $installed_ver = get_option( DiviManagerPro::VERSION_NAME );

        if ( $installed_ver != DiviManagerPro::VERSION ) {
            // set some defaults
            $optionService = $this->registry->getService(Option::class);
            $options = $optionService->getOptions();
            $options['report_email'] = \get_option( 'admin_email' );
            $options['report_cc'] = '';
            $options['report_subject'] = '[dm.wp.site.name] Status Report for [dm.report.date.month]';
            $templateFile = WP_PLUGIN_DIR . '/divimanager-pro/templates/email.phtml';
            //$templateFile = __DIR__ . '/../templates/email.phtml';
            $options['report_template'] = file_get_contents($templateFile);
            $options['report_last_run'] = '';
            $optionService->setOptions($options);
        }

        //update_option( self::VERSION_NAME, self::VERSION );
    }

    /**
     * Need to let wordpress know what query variables we use
     */
    public function addQueryVars( $vars )
    {
        $vars[] = 'site_id';
        $vars[] = 'slug';

        return $vars;
    }

    /**
     * End setup/hooks
     */

    /**
     * return the target month
     */
    public function getTargetMonth()
    {
        return $this->targetMonth;
    }

    /**
     * set the target month for reports
     */
    public function setTargetMonth(\DateTime $target)
    {
        $this->targetMonth = $target;
    }

    /**
     * Return the short code prefix used to namespace our shortcodes
     */
    public function getScPrefix()
    {
        return self::SC_PREFIX;
    }

    /**
     * Whether the Reporter functionality is enabled
     *
     * @return bool
     */
    public function isReporterEnabled()
    {
        return true === $this->option->getOption('reporter_enabled');
    }
}
