<?php declare(strict_types=1);
namespace DiviManagerPro\Rest;

defined( 'ABSPATH' ) or die( 'Unexpected access' );

use DiviManager\Helper\Option;
use DiviManagerPro\Service\Integration as IntegrationService;
use DiviManagerPro\Integration\Setting as IntegrationSetting;
use WP_REST_Request;
use WP_REST_Server;
use WP_REST_Controller;

/**
 * Integration REST api support
 *
 * @see https://developer.wordpress.org/rest-api/extending-the-rest-api/adding-custom-endpoints/
 * @see https://torquemag.io/2017/06/creating-wordpress-settings-page-using-wordpress-rest-api/
 */
class Integration extends WP_REST_Controller
{
    /**
     * @var String
     */
    protected $base;

    /**
     * @var IntegrationSetting
     */
    protected $integrationSettings;

    public function __construct($base, IntegrationService $integrationService, IntegrationSetting $integrationSettings)
    {
        $this->setBase($base);
        $this->setIntegrationService($integrationService);
        $this->setIntegrationSettings($integrationSettings);
    }

    public function getBase()
    {
        return $this->base;
    }

    public function setBase($base)
    {
        $this->base = $base;
    }

    public function getIntegrationService()
    {
        return $this->integrationService;
    }

    public function setIntegrationService(IntegrationService $integrationService)
    {
        $this->integrationService = $integrationService;
    }

    public function getIntegrationSettings()
    {
        return $this->integrationSettings;
    }

    public function setIntegrationSettings(IntegrationSetting $integrationSettings)
    {
        $this->integrationSettings = $integrationSettings;
    }

    public function init()
    {
        add_action( 'rest_api_init', [$this, 'register_rest_routes']);
    }

    /**
     * Register our rest routes with the REST endpoint
     */
    public function register_rest_routes()
    {
        // get settings
        register_rest_route( $this->base, '/settings', [
            'methods' => WP_REST_Server::READABLE,
            'callback' => [$this, 'getList'],
            'args' => [
                'integration' => [
                    'type' => 'string',
                    'required' => false,
                    'sanitize_callback' => 'sanitize_text_field'
                ],
            ],
            'permission_callback' => [$this, 'permissions'],
        ]);
        // post settings
        register_rest_route( $this->base, '/settings', [
            'methods' => WP_REST_Server::CREATABLE,
            'callback' => [$this, 'update'],
            'args' => [
                'industry' => [
                    'type' => 'string',
                    'required' => false,
                    'sanitize_callback' => 'sanitize_text_field'
                ],
                'amount' => [
                    'type' => 'integer',
                    'required' => false,
                    'sanitize_callback' => 'absint'
                ]
            ],
            'permission_callback' => [$this, 'permissions'],
        ]);

        register_rest_route( $this->base, '/(?P<type>\w+)', [
            'methods' => WP_REST_Server::READABLE,
            'callback' => [$this, 'getListByType'],
            'args' => [
                'integration' => [
                    'type' => 'string',
                    'required' => false,
                    'sanitize_callback' => 'sanitize_text_field'
                ],
            ],
            'permission_callback' => [$this, 'permissions'],
        ]);
    }

    /**
     * Return the settings and their values
     */
    public function getList(WP_REST_Request $request)
    {
        $integrationID = $request->get_param('integration');
        $integration = $this->integrationService->integration($integrationID);
        if (null == $integration) {
            return rest_ensure_response( null );
        }
        return rest_ensure_response( $integration->getSettings() );
        //return rest_ensure_response( $this->integrationSettings->getSettings());
    }

    /**
     * Return the integration services for a type
     */
    public function getListByType(WP_REST_Request $request)
    {
        $type = sanitize_text_field( $request->get_param('type') );
        $integrations = $this->integrationService->integrationsWithType($type);
        $response = [];
        foreach ($integrations as $integration) {
            $response[$integration->getId()] = $integration->getName();
        }
        return rest_ensure_response( $response );
    }

    /**
     * Update setting values
     */
    public function update(WP_REST_Request $request)
    {
        $integrationID = $request->get_param('integration');
        $integration = $this->integrationService->integration($integrationID);
        $integration->updateSettings($request->get_params());
        return rest_ensure_response( $integration->getSettings() );
    }

    /**
     * Check if access is allowed on the endpoint
     *
     * @return bool
     */
    public function permissions()
    {
        return true;
        // make sure the user can manage options
        return current_user_can( 'manage_options' );
    }
}
