<?php declare(strict_types=1);
namespace DiviManagerPro\Rest;

defined( 'ABSPATH' ) or die( 'Unexpected access' );

use DiviManager\Helper\Option;
use DiviManager\Service\Profile as ProfileService;
use DiviManagerPro\Service\Integration as IntegrationService;
use DiviManagerPro\Integration\AbstractIntegration as Integration;
use DiviManagerPro\Service\Server as ServerService;
use WP_REST_Request;
use WP_REST_Server;
use WP_REST_Controller;
use WP_REST_Response;

/**
 * Profile REST api support
 */
class Profile extends WP_REST_Controller
{
    /**
     * @var String
     */
    protected $base;

    /**
     * @var ProfileService
     */
    protected $profileService;

    /**
     * @var IntegrationService
     */
    protected $integrationService;

    public function __construct($base, ProfileService $profileService, IntegrationService $integrationService, $formApi, $theme, $plugin, $profilePlugin, $profileIntegrationService, $profileServerService, $serverService)
    {
        $this->setBase($base);
        $this->setProfileService($profileService);
        $this->setIntegrationService($integrationService);
        $this->themeService = $theme;
        $this->pluginService = $plugin;
        $this->profilePluginService = $profilePlugin;
        $this->profileIntegrationService = $profileIntegrationService;
        $this->profileServerService = $profileServerService;
        $this->serverService = $serverService;
        $formApi->addForm('profile', $this);

        $this->settings = [
            'name' => [
                'name' => 'Name',
                'description' => 'Name of the profile.',
                'id' => 'name',
                'type' => 'text',
                'validation' => null,
            ],
            'description' => [
                'name' => 'Description',
                'description' => 'A description of this profile.',
                'id' => 'description',
                'type' => 'textarea',
                'validation' => null,
            ],
            'dns' => [
                'name' => 'DNS Service',
                'description' => 'The dns service to use.  If left blank, this profile will not add a subdomain to any existing tld.',
                'id' => 'dns',
                'type' => 'select',
                'validation' => null,
                'route' => esc_url_raw( rest_url( 'dmpro/v1/integration/dns' ) ),
            ],
            'server' => [
                'name' => 'Hosting Server',
                'description' => 'The hosting server on which the site will be created.  If left blank, no site will be created.',
                'id' => 'host',
                'type' => 'select',
                'validation' => null,
                'route' => esc_url_raw( rest_url( 'dmpro/v1/servers' ) ),
            ],
            'theme' => [
                'name' => 'Theme',
                'description' => 'Select the theme which this profile prefers.',
                'id' => 'theme',
                'type' => 'select',
                'validation' => null,
                'route' => esc_url_raw( rest_url( 'dm/v1/themes' ) ),
            ],
            'plugin' => [
                'name' => 'Plugins',
                'description' => 'Check plugins which should be included in this profile.',
                'id' => 'plugin',
                'type' => 'multicheckbox',
                'validation' => null,
                'route' => esc_url_raw( rest_url( 'dm/v1/plugins' ) ),
            ],
        ];
    }

    public function getBase()
    {
        return $this->base;
    }

    public function setBase($base)
    {
        $this->base = $base;
    }

    public function getProfileService()
    {
        return $this->profileService;
    }

    public function setProfileService(ProfileService $profileService)
    {
        $this->profileService = $profileService;
    }

    public function getIntegrationService()
    {
        return $this->integrationService;
    }

    public function setIntegrationService(IntegrationService $integrationService)
    {
        $this->integrationService = $integrationService;
    }

    public function init()
    {
        add_action( 'rest_api_init', [$this, 'register_rest_routes']);
    }

    /**
     * Register our rest routes with the REST endpoint
     */
    public function register_rest_routes()
    {
        register_rest_route( $this->base, '/profile', [
            'methods' => WP_REST_Server::READABLE,
            'callback' => [$this, 'getList'],
            'permission_callback' => [$this, 'permissions'],
        ]);
        register_rest_route( $this->base, '/profile/(?P<id>[\d]+)', [
            [
                'methods' => WP_REST_Server::READABLE,
                'callback' => [$this, 'get'],
                'args' => [
                    'profile' => [
                        'type' => 'string',
                        'required' => false,
                        'sanitize_callback' => 'sanitize_text_field'
                    ],
                ],
                'permission_callback' => [$this, 'permissions'],
            ],
            [
                'methods' => WP_REST_Server::DELETABLE,
                'callback' => [$this, 'delete'],
                'args' => [
                    'profile' => [
                        'type' => 'string',
                        'required' => false,
                        'sanitize_callback' => 'sanitize_text_field'
                    ],
                ],
                'permission_callback' => [$this, 'permissions'],
            ],
            // post settings
            [
                'methods' => WP_REST_Server::EDITABLE,
                'callback' => [$this, 'update'],
                'args' => [
                    'industry' => [
                        'type' => 'string',
                        'required' => false,
                        'sanitize_callback' => 'sanitize_text_field'
                    ],
                    'amount' => [
                        'type' => 'integer',
                        'required' => false,
                        'sanitize_callback' => 'absint'
                    ]
                ],
                'permission_callback' => [$this, 'permissions'],
            ]
        ]);
        register_rest_route( $this->base, '/profile', [
            'methods' => WP_REST_Server::CREATABLE,
            'callback' => [$this, 'create'],
            'permission_callback' => [$this, 'permissions'],
        ]);
        register_rest_route( $this->base, '/(?P<id>[\d]+)', [
            'methods' => WP_REST_Server::READABLE,
            'callback' => [$this, 'update'],
            'args' => [
                'profile' => [
                    'type' => 'string',
                    'required' => false,
                    'sanitize_callback' => 'sanitize_text_field'
                ],
            ],
            'permission_callback' => [$this, 'permissions'],
        ]);
    }

    /**
     * Return the form and their values
     */
    public function getForm()
    {
        $settings = $this->settings;
        return rest_ensure_response( $settings );
        // need to add dns services
        $settings['dns']['options'] = [];
        $dnsServices = $this->integrationService->integrationsWithType(Integration::SERVICE_DNS);
        foreach ($dnsServices as $service) {
            $settings['dns']['options'][] = [
                'value' => $service->getId(),
                'label' => $service->getName()
            ];
        }
        // need to add hosting services
        $settings['host']['options'] = [];
        $hostServices = $this->integrationService->integrationsWithType(Integration::SERVICE_HOST);
        foreach ($hostServices as $service) {
            foreach ($service->getHostOptions() as $option) {
                $settings['host']['options'][] = [
                    'value' => $option['id'],
                    'label' => $option['name']
                ];
            }
        }

        return rest_ensure_response( $settings );
    }

    /**
     * Return the list of profiles
     */
    public function getList(WP_REST_Request $request)
    {
        $profiles = [];
        foreach ($this->profileService->getProfiles() as $profile) {
            $viewUrl = admin_url( 'admin.php?page=divimanager-profile-view&profile_id=' . $profile->getId() );
            //$viewLinkUrl = sprintf($viewLink, $viewUrl, $profile->getName());
            $profiles[] = [
                'id' => $profile->getId(),
                'name' => $profile->getName(),
                'description' => $profile->getDescription(),
                '_links' => [
                    'self' => [
                        'href' => $viewUrl,
                    ],
                ],
            ];
        }
        return rest_ensure_response( $profiles );
    }

    /**
     * Return the information for a profile
     */
    public function get(WP_REST_Request $request)
    {
        $profileID = sanitize_text_field( $request->get_param( 'id' ) );
        $profile = $this->profileService->getProfile($profileID);
        if (null === $profile) {
            throw new \Exception('Profile not found for id:' . $profileID);
        }
        $values = $profile->toArray();
        $values['plugins'] = [];
        $plugins = $this->profilePluginService->getPluginsForProfile($profile);
        foreach ($plugins as $plugin) {
            $values['plugins'][] = $plugin->getId();
        }
        $integrations = $this->profileIntegrationService->getIntegrationsForProfile($profile);
        $values['dns'] = null;
        foreach ($integrations as $integration) {
            if ($integration::SERVICE_DNS == $integration->getType()) {
                $values['dns'] = $integration->getId();
                break;
            }
        }
        $values['server'] = null;
        $server = $this->profileServerService->getServerForProfile($profile);
        if (! empty($server) && null !== $server[0]) {
            $values['server'] = $server[0]->getId();
        }

        return rest_ensure_response( $values );
    }

    /**
     * Delete a profile
     */
    public function delete(WP_REST_Request $request)
    {
        $profileID = sanitize_text_field( $request->get_param( 'id' ) );
        $profile = $this->profileService->getProfile($profileID);
        if (null === $profile) {
            return new \WP_REST_Response(false);
            throw new \Exception('Profile not found for id:' . $profileID);
        }

        $plugins = $this->profilePluginService->getPluginsForProfile($profile);
        foreach ($plugins as $plugin) {
            $this->profilePluginService->removePluginFromProfile($plugin, $profile);
        }

        $integrations = $this->profileIntegrationService->getIntegrationsForProfile($profile);
        foreach ($integrations as $integration) {
            $this->profileIntegrationService->removeIntegrationFromProfile($integration, $profile);
        }

        $server = $this->profileServerService->getServerForProfile($profile);
        if (null !== $server && ! empty($server)) {
            $this->profileServerService->removeServerFromProfile($server[0], $profile);
        }

        $this->profileService->deleteProfile($profile);

        return new \WP_REST_Response(true);
    }

    /**
     * Create a new profile
     */
    public function create(WP_REST_Request $request)
    {
        $profile = $this->profileService->newModel();
        $profile->setName(sanitize_text_field( $request->get_param('name') ));
        $profile->setDescription(sanitize_text_field( $request->get_param('description') ));
        $profile->setThemeID(sanitize_text_field( $request->get_param( 'theme' ) ));
        $profile = $this->profileService->createProfile($profile);
        foreach ($request->get_param('plugins') as $plugin) {
            $plugin = $this->pluginService->getPlugin(sanitize_text_field( $plugin ));
            $this->profilePluginService->addPluginToProfile($plugin, $profile);
        }
        // dns service
        $integration = $this->integrationService->integration(sanitize_text_field( $request->get_param('dns') ));
        if (null !== $integration) {
            $this->profileIntegrationService->addIntegrationToProfile($integration, $profile);
        }
        // host service
        $integration = $this->integrationService->integration(sanitize_text_field( $request->get_param('host') ));
        if (null !== $integration) {
            $this->profileIntegrationService->addIntegrationToProfile($integration, $profile);
        }
    }

    /**
     * Update setting values
     */
    public function update(WP_REST_Request $request)
    {
        $id = sanitize_text_field( $request->get_param('id') );
        $profile = $this->profileService->getProfile($id);
        if (null === $profile) {
            return new \WP_REST_Response(false);
        }
        $profile->setName(sanitize_text_field( $request->get_param('name') ));
        $profile->setDescription(sanitize_text_field( $request->get_param('description') ));
        $profile->setThemeID(sanitize_text_field( $request->get_param( 'theme' ) ));
        $profile = $this->profileService->updateProfile($profile);
        foreach ($request->get_param('plugins') as $plugin) {
            $plugin = $this->pluginService->getPlugin(sanitize_text_field( $plugin ));
            $this->profilePluginService->addPluginToProfile($plugin, $profile);
        }
        // dns service
        $integration = $this->integrationService->integration(sanitize_text_field( $request->get_param('dns') ));
        if (null !== $integration) {
            $this->profileIntegrationService->addIntegrationToProfile($integration, $profile);
        }
        // host service
        $server = $this->serverService->getServer((int) sanitize_text_field( $request->get_param('host') ));
        if (null !== $server) {
            $this->profileServerService->addServerToProfile($server, $profile);
        }

        return new \WP_REST_Response(true);
    }

    /**
     * Check if access is allowed on the endpoint
     *
     * @return bool
     */
    public function permissions()
    {
        // make sure the user can manage options
        return current_user_can( 'manage_options' );
    }
}
