<?php declare(strict_types=1);
namespace DiviManagerPro\Rest;

defined( 'ABSPATH' ) or die( 'Unexpected access' );

use DiviManagerPro\Service\Server as ServerService;
use DiviManagerPro\Service\Integration;
use DiviManagerPro\Service\IntegrationServer;

/**
 * REST api for managing servers
 */
class Server
{
    /**
     * @var String
     */
    protected $base;

    /**
     * @var ServerService
     */
    protected $serverService;

    /**
     * @var Integration
     */
    protected $integrationService;

    /**
     * @var IntegrationServer
     */
    protected $integrationServerService;

    public function __construct(
        $base,
        ServerService $serverService,
        IntegrationServer $integrationServerService,
        Integration $integrationServer,
    ) {
        $this->setBase($base);
        $this->setServerService($serverService);
        $this->setIntegrationServerService($integrationServerService);
        $this->setIntegrationService($integrationServer);
    }

    public function getBase()
    {
        return $this->base;
    }

    public function setBase($base)
    {
        $this->base = $base;
    }

    public function setServerService(ServerService $s)
    {
        $this->serverService = $s;
    }

    public function setIntegrationServerService(IntegrationServer $s)
    {
        $this->integrationServerService = $s;
    }

    public function setIntegrationService(Integration $s)
    {
        $this->integrationService = $s;
    }

    public function init()
    {
        add_action( 'rest_api_init', [$this, 'register_rest_routes']);
    }

    public function register_rest_routes()
    {
        // server routes
        register_rest_route( $this->base, '/servers', [
            [
                'methods' => \WP_REST_Server::READABLE,
                'callback' => [$this, 'getList'],
                'permission_callback' => [$this, 'verifyToken'],
            ],
            [
                'methods' => \WP_REST_Server::CREATABLE,
                'callback' => [$this, 'create'],
                'permission_callback' => [$this, 'verifyToken'],
            ]
        ]);
        register_rest_route( $this->base, '/servers/(?P<id>[\d]+)', [
            [
                'methods' => \WP_REST_Server::READABLE,
                'callback' => [$this, 'get'],
                'permission_callback' => [$this, 'verifyToken'],
            ],
            [
                'methods' => \WP_REST_Server::DELETABLE,
                'callback' => [$this, 'deleteServer'],
                'permission_callback' => [$this, 'verifyToken'],
            ]
        ]);
        register_rest_route( $this->base, '/server/upgrade', [
            'methods' => \WP_REST_Server::CREATABLE,
            'callback' => [$this, 'upgrade'],
            'permission_callback' => [$this, 'verifyToken'],
        ]);
    }

    /**
     * Check that the request has a valid token
     *
     * @return null|WP_Error null if valid, WP_Error if invalid
     */
    public function verifyToken($request = null)
    {
        // from wp-includes/rest-api/endpoints/class-wp-rest-plugins-controller.php
        //if ( ! current_user_can( 'activate_plugins' ) ) {
        //if ( ! current_user_can( 'manage_options' ) ) {
        if ( ! current_user_can( 'administrator' ) ) {
            return new \WP_Error(
                'rest_cannot_view_servers',
                __( 'Sorry, you are not allowed to manage servers for this site.' ),
                array( 'status' => rest_authorization_required_code() )
            );
        }

        return true;
    }

    /**
     * Return the servers
     */
    public function getList($request)
    {
        $host = sanitize_text_field( $request->get_param('host') );
        if (null === $host || empty($host)) {
            // no host specified, get all servers
            $servers = $this->serverService->getRecords();
        } else {
            // get only servers for the matching host
            $integration = $this->integrationService->integration($host);
            if (null === $integration) {
                throw new \Exception('No integration service matching: ' . $host); 
            }
            $servers = $this->integrationServerService->getServersForIntegration($integration);
        }

        $data = [];
        foreach ($servers as $server) {
            $data[] = $server->toArray();
        }

        return new \WP_REST_Response($data);
    }

    /**
     * Get a specific server
     */
    public function get($request)
    {
        var_dump(__FUNCTION__, $request);
    }

    /**
     * Add a new server
     */
    public function create($request)
    {
        $params = $request->get_params();
        $server = $this->serverService->newModel();
        $server->setName($params['name']);
        $server->setIp($params['ip']);
        $server->setIpv6($params['ipv6']);
        $server->setServerID($params['serverID']);
        $server = $this->serverService->createServer($server);
        return new \WP_REST_Response($server->toArray());
    }

    /**
     * Delete an existing server
     */
    public function deleteServer($request)
    {
        $params = $request->get_params();
        $server = $this->serverService->getRecord($params['id']);

        if (null === $server) {
            return new \WP_REST_Response(false);
        }

        $this->serverService->deleteServer($server);
        return new \WP_REST_Response(true);
    }

    public function activate($request)
    {
        // activate the plugin
        include_once ABSPATH . 'wp-admin/includes/plugin.php'; // for activate_plugin
        $result = \activate_plugins( $_POST['slug'] );
        return new \WP_REST_Response($result);
    }

    public function deactivate($request)
    {
        include_once ABSPATH . 'wp-admin/includes/plugin.php'; // for deactivate_plugin
        $result = \deactivate_plugins( $_POST['slug'] );
        return new \WP_REST_Response($result);
    }

    /**
     * Upgrade a plugin
     * 
     * see wp-admin/includes/ajax-actions.php
     */
    public function upgrade($request)
    {
        // setup
        include_once ABSPATH . 'wp-admin/includes/file.php';
        include_once ABSPATH . 'wp-admin/includes/plugin-install.php'; // For plugins_api()
        require_once ABSPATH . 'wp-admin/includes/update.php';
        require_once ABSPATH . 'wp-admin/includes/plugin.php';
        require_once ABSPATH . 'wp-admin/includes/class-wp-upgrader.php';
        include_once ABSPATH . 'wp-admin/includes/class-plugin-upgrader.php';
        //include_once ABSPATH . 'wp-admin/includes/class-class-wp-ajax-upgrader-skin.php';
        // this is for aio-seo...
        if (class_exists('AIOSEO\Plugin\Pro\Admin\Updates')) {
            $aioseoUpdate = new \AIOSEO\Plugin\Pro\Admin\Updates([
                        'pluginSlug' => 'all-in-one-seo-pack-pro',
                        'pluginPath' => plugin_basename( AIOSEO_FILE ),
                        'version'    => AIOSEO_VERSION,
                        'key'        => aioseo()->options->general->licenseKey
            ]);
            add_filter( 'pre_set_site_transient_update_plugins', [ $aioseoUpdate, 'updatePluginsFilter' ], 1000 );
        }       

        // clear the plguin and updates cache
        //\delete_site_transient( 'update_plugins' );
        \wp_update_plugins();
        $plugins = get_plugins();
        $updates = get_site_transient( 'update_plugins' );
        //\wp_clean_plugins_cache(true);
        // wp ajax handles nonce checking automatically?
        $skin     = new \WP_Ajax_Upgrader_Skin();
        $upgrader = new \Plugin_Upgrader( $skin );

        $slug = $_POST['slug'];
        $files = $request->get_file_params();
        $status = [
            'update'     => 'plugin',
            'slug'       => $slug,
            'oldVersion' => '',
            'newVersion' => '',
        ];

        // this needs redone?
        // check the slug first
        // check if the plugin can be upgraded normally
        // then, check if the file is being sent?
        if (! empty($files)) {
            $file = $files['archive'];
            $this->addToTransient($slug, $file['tmp_name']);
            $result = $upgrader->upgrade( $slug );
            if (\is_wp_error($result)) {
                return $result;
            }
        } else {
            //$result = $upgrader->upgrade( $slug );
            $result   = $upgrader->bulk_upgrade( [ $slug ] );
            if ( is_wp_error( $skin->result ) ) {
                $status['errorCode']    = $skin->result->get_error_code();
                $status['errorMessage'] = $skin->result->get_error_message();
                wp_send_json_error( $status );
            } elseif ( $skin->get_errors()->has_errors() ) {
                $status['errorMessage'] = $skin->get_error_messages();
                wp_send_json_error( $status );
            } elseif ( is_array( $result ) && ! empty( $result[ $slug ] ) ) {
                // need to activate the plugin as well.
                $activated = activate_plugin( $slug, '' );
                if ( is_wp_error( $activated ) ) {
                    $activated->add_data( [ 'status' => 500 ] );

                    return $activated;
                } else {
                    wp_send_json_success( $status );
                }
            } elseif ( false === $result ) {
                wp_send_json_error( $status );
            }

            $status['errorMessage'] = __( 'Plugin update failed.' );
            wp_send_json_error( $status );
        }

        return new \WP_REST_Response($result);
    }
}
