<?php declare(strict_types=1);
namespace DiviManagerPro\Service;

use DiviManager\Service\DbManager;
use DiviManagerPro\Model\Server as ServerModel;
use DiviManagerPro\Integration\IntegrationInterface as IntegrationModel;
use DiviManagerPro\Service\Server as ServerService;
use DiviManagerPro\Service\Integration as IntegrationService;

/**
 * Service class for working with integration server joins
 */
class IntegrationServer extends DbManager
{
    /**
     * @var IntegrationService
     */
    protected $integrationService;

    /**
     * @var ServerService;
     */
    protected $serverService;

    public function __construct()
    {
        $fields =<<<EOSQL
integration_id varchar(64) null,
server_id int NOT NULL
EOSQL;

        parent::__construct('divimanager_pro_integrations_servers', $fields);
    }

    /**
     * Set the integration service
     */
    public function setIntegrationService(IntegrationService $p)
    {
        $this->integrationService = $p;
    }

    /**
     * Set the server service
     */
    public function setServerService(ServerService $s)
    {
        $this->serverService = $s;
    }

    /**
     * Get servers for given integration
     *
     * @param Integration
     * @return array
     */
    public function getServersForIntegration( IntegrationModel $integration )
    {
        $ids = $this->getFieldByField('server_id', ['integration_id' => $integration->getID()]);

        if (null === $ids) {
            return [];
        }

        return $this->serverService->getServersByID($ids);
    }

    /**
     * Get servers for multiple integrations
     *
     * @param integration
     * @return array
     */
    public function getServersForIntegrations( array $integrations = [], $options = null )
    {
        if (empty($integrations)) {
            return [];
        }

        $integrationIDs = [];
        foreach ($integrations as $integration) {
            $integrationIDs[] = $integration->getId();
        }

        $ids = $this->getFieldByField('server_id', ['integration_id' => $integrationIDs]);

        if (null === $ids) {
            return [];
        }

        return $this->serverService->getServersByID($ids);
    }

    /**
     * Get integration for given server
     *
     * @param Server
     * @return Integration
     */
    public function getIntegrationForServer( ServerModel $server )
    {
        var_dump(__FUNCTION__, $server->getID());
        $ids = $this->getFieldByField('integration_id', ['server_id' => $server->getID()]);

        var_dump(__FUNCTION__, $ids);
        if (null === $ids || empty($ids)) {
            return [];
        }

        return $this->integrationService->integration($ids[0]);
    }

    /**
     * Add server to integration
     *
     * @param ServerModel
     * @param IntegrationModel
     */
    public function addServerToIntegration( ServerModel $server, IntegrationModel $integration)
    {
        $this->wpdb->insert(
            $this->getTableName(),
            [
                'integration_id' => $integration->getId(),
                'server_id' => $server->getId(),
            ],
            [
                '%s',
                '%s',
            ]
        );
    }

    /**
     * Remove server from integration
     *
     * @param ServerModel
     * @param IntegrationModel
     */
    public function removeServerFromIntegration( ServerModel $server, IntegrationModel $integration)
    {
        $this->wpdb->delete(
            $this->tableName,
            [
                'integration_id' => $integration->getId(),
                'server_id' => $server->getId(),
            ],
            [
                '%s',
                '%s',
            ]
        );
    }
}
