<?php declare(strict_types=1);
namespace DiviManagerPro\Service;

use DiviManager\Service\DbManager;
use DiviManager\Model\Profile as ProfileModel;
use DiviManager\Service\Profile as ProfileService;
use DiviManagerPro\Service\Integration as IntegrationService;

/**
 * Service class for working with profile integration service joins
 */
class ProfileIntegration extends DbManager
{
    /**
     * @var ProfileService
     */
    protected $profileService;

    /**
     * @var IntegrationService
     */
    protected $integrationService;

    public function __construct(
        $wpdb,
        $dbDelta,
        ProfileService $profileService,
        IntegrationService $integrationService
    ) {
        $fields =<<<EOSQL
profile_id varchar(64) NOT NULL,
integration_id varchar(64) NOT NULL
EOSQL;

        parent::__construct('divimanager_pro_profile_integration', $fields);
        $this->setWpdb($wpdb);
        $this->setDbDelta($dbDelta);
        $this->setProfileService($profileService);
        $this->setIntegrationService($integrationService);
    }

    /**
     * Set the profile service
     */
    public function setProfileService(ProfileService $p)
    {
        $this->profileService = $p;
    }

    /**
     * Set the integration service
     */
    public function setIntegrationService(IntegrationService $s)
    {
        $this->integrationService = $s;
    }

    /**
     * Get integrations for the profile
     *
     * @param ProfileModel
     * @return array
     */
    public function getIntegrationsForProfile(ProfileModel $profile)
    {
        $ids = $this->getFieldByField('integration_id', ['profile_id' => $profile->getID()]);

        if (null === $ids) {
            return [];
        }

        return $this->integrationService->getIntegrationsByID($ids);
    }

    /**
     * Get integration for the profile, limited by type
     *
     * Will return the first matching integration
     * Not sure how likely multiple integrations of the same type will be associated with a single profile
     *
     * @param ProfileModel
     * @return IntegrationService
     */
    public function getIntegrationForProfileByType(ProfileModel $profile, $type)
    {
        $ids = $this->getFieldByField('integration_id', ['profile_id' => $profile->getID()]);

        if (null === $ids) {
            return [];
        }

        // Doing this as a db query would be nice, instead of a loop
        $integrations = $this->integrationService->getIntegrationsByID($ids);
        foreach ($integrations as $integration) {
            if ($integration::SERVICE_DNS == $integration->getType()) {
                break;
            }
        }

        return $integration;
    }

    /**
     * Get profiles with the integration
     *
     * @param Integration
     * @return array
     */
    public function getProfilesForIntegration($integration)
    {
        $ids = $this->getFieldByField('profile_id', ['integration_id' => $integration->getID()]);

        if (null === $ids) {
            return [];
        }

        return $this->profileService->getProfilesByIDs($ids);
    }

    /**
     * Add integration to profile
     *
     * @param Integration
     * @param ProfileModel
     */
    public function addIntegrationToProfile($integration, ProfileModel $profile)
    {
        $this->wpdb->insert(
            $this->getTableName(),
            [
                'profile_id' => $profile->getId(),
                'integration_id' => $integration->getId(),
            ],
            [
                '%s',
                '%s',
            ]
        );
    }

    /**
     * Remove integration from profile
     *
     * @param Integration
     * @param ProfileModel
     */
    public function removeIntegrationFromProfile($integration, ProfileModel $profile)
    {
        $this->wpdb->delete(
            $this->getTableName(),
            [
                'profile_id' => $profile->getId(),
                'integration_id' => $integration->getId(),
            ],
            [
                '%s',
                '%s',
            ]
        );
    }
}
