<?php declare(strict_types=1);
namespace DiviManagerPro\Woocommerce;

defined( 'ABSPATH' ) or die( 'Unexpected access' );

use DiviManager\Model\Site as SiteModel;
use DiviManager\Service\Site as SiteService;

/**
 * Add custom field to products
 */
class Field
{
    /**
     * Woocommerce product field
	 * see https://www.kathyisawesome.com/add-a-custom-field-to-woocommerce-product/
     * see https://catapultthemes.com/add-custom-fields-woocommerce-product/
     */

    /**
     * @var DiviManager\Woocommerce\Woocommerce
     */
    protected $dmpwc;

    /**
     * @var String
     */
    protected $fieldName;

    /**
     * @var String
     */
    protected $fieldLabel;

    /**
     * @var SiteService
     */
    protected $siteService;

    public function __construct(Woocommerce $dmpwc, $fieldName, $fieldLabel, $siteService)
    {
        $this->dmpwc = $dmpwc;
        $this->fieldName = $fieldName;
        $this->fieldLabel = $fieldLabel;
        $this->siteService = $siteService;
    }

	/**
	 * Display input on single product page
	 * @return html
	 */
    public function add_custom_input()
    {
        global $post;

		$product = wc_get_product( $post->ID );

        $profile = $this->dmpwc->product_matches_profile($product);
        if (false === $profile) {
            // not a provision
            return false;
        }

        $value = isset( $_POST[$this->fieldName] ) ? sanitize_text_field( $_POST[$this->fieldName] ) : '';
        printf( '<div class="wcfield"><label>%s</label><input name="%s" value="%s" /></div>', __( $this->fieldLabel ), $this->fieldName, esc_attr( $value ) );
	}

	/**
	 * Validate field when adding to cart
	 * @param bool $passed
	 * @param int $product_id
	 * @param int $quantity
	 * @return bool
	 */
    public function add_to_cart_validation($passed, $product_id, $qty)
    {
		$product = wc_get_product( $product_id );

        $profile = $this->dmpwc->product_matches_profile($product);
        if (false === $profile) {
            // not a provision
            return $passed;
        }

        if( isset( $_POST[$this->fieldName] ) ){
            $sanitized = sanitize_text_field($_POST[$this->fieldName]);
            if (null === $sanitized || false === $sanitized || '' === $sanitized) {
                wc_add_notice( sprintf( __( '%s requires %s'), $product->get_title(), $this->fieldLabel ), 'error' );
                return false;
            } else {
                return $passed;
            }
        } else {
            // required field
            return false;
        }
	}

     /**
	 * Add custom data to the cart item
	 * @param array $cart_item
	 * @param int $product_id
	 * @return array
	 */
    public function add_cart_item_data( $cart_item, $product_id )
    {
        $product = wc_get_product( $product_id );

        $profile = $this->dmpwc->product_matches_profile($product);
        if (false === $profile) {
            // not a provision
            return $cart_item;
        }

        if( isset( $_POST[$this->fieldName] ) ) {
            $cart_item[$this->fieldName] = sanitize_text_field( $_POST[$this->fieldName] );
        }

		return $cart_item;
	}

	/**
	 * Load cart data from session
	 * @param array $cart_item
	 * @param array $other_data
	 * @return array
	 */
    public function get_cart_item_from_session( $cart_item, $values )
    {
        if ($cart_item instanceof \WC_Product) {
            $product = $cart_item;
        } elseif (is_array($cart_item) && isset($cart_item['product_id'])) {
            $product = wc_get_product( $cart_item['product_id'] );
        } else {
            return $cart_item;
        }

        $profile = $this->dmpwc->product_matches_profile($product);
        if (false === $profile) {
            // not a provision
            return $cart_item;
        }

        if ( isset( $values[$this->fieldName] ) ){
            $cart_item[$this->fieldName] = $values[$this->fieldName];
        }

		return $cart_item;
	}

	/**
     * Add meta to order item
     * see https://stackoverflow.com/questions/45706007/get-order-items-and-wc-order-item-product-in-woocommerce-3
     * @param $item
     * @param $cart_item_key
     * @param array $values
     * @param order
	 * @return void
	 */
    public function checkout_create_order_line_item( $item, $cart_item_key, $values, $order )
    {
        if (! $item instanceof \WC_Order_Item_Product) {
            // not an order item
            return;
        }

        // The product ID
        $product = $item->get_product();

        $profile = $this->dmpwc->product_matches_profile($product);
        if (false === $profile) {
            // not a provision
            return;
        }

        if ( ! empty( $values[$this->fieldName] ) ) {
            $item->update_meta_data( $this->fieldName, $values[$this->fieldName] );
        } else {
        }
	}

	/**
	 * Get item data to display in cart
	 * @param array $other_data
	 * @param array $cart_item
	 * @return array
	 */
    public function get_item_data( $other_data, $cart_item)
    {
        if ($cart_item instanceof \WC_Product) {
            $product = $cart_item;
        } elseif (is_array($cart_item) && isset($cart_item['product_id'])) {
            $product = wc_get_product( $cart_item['product_id'] );
        } else {
            return $other_data;
        }

        $profile = $this->dmpwc->product_matches_profile($product);
        if (false === $profile) {
            // not a provision
            return $other_data;
        }

        if ( isset( $cart_item[$this->fieldName] ) ){

            $other_data[] = [
                'name' => __( $this->fieldLabel ),
                'value' => sanitize_text_field( $cart_item[$this->fieldName] )
            ];
        }

		return $other_data;
	}

	/**
	 * Show custom field in order overview
	 * @param array $cart_item
	 * @param array $order_item
	 * @return array
	 */
    public function order_item_product( $cart_item, $order_item )
    {
        if ($cart_item instanceof \WC_Product) {
            $product = $cart_item;
        } elseif (is_array($cart_item) && isset($cart_item['product_id'])) {
            $product = wc_get_product( $cart_item['product_id'] );
        } else {
            return $cart_item;
        }

        $profile = $this->dmpwc->product_matches_profile($product);
        if (false === $profile) {
            // not a provision
            return $cart_item;
        }

        $cart_item->update_meta_data($this->fieldName, $order_item->get_meta_data($this->fieldName));

		return $cart_item;
	}

	/* *
	 * Add the field to order emails 
	 * @param array $keys 
	 * @return array 
	 */ 
    public function email_order_meta_fields( $fields, $sent_to_admin, $order)
    { 
        $fields[$this->fieldLabel] = $this->fieldName;

		return $fields; 
	} 

	/**
	 * Order Again
	 * @param array $cart_item
	 * @param array $order_item
	 * @param obj $order
	 * @return array
	 */
    public function order_again_cart_item_data( $cart_item, $order_item, $order )
    {
        if ($cart_item instanceof \WC_Product) {
            $product = $cart_item;
        } elseif (is_array($cart_item) && isset($cart_item['product_id'])) {
            $product = wc_get_product( $cart_item['product_id'] );
        } else {
            return $cart_item;
        }

        $profile = $this->dmpwc->product_matches_profile($product);
        if (false === $profile) {
            // not a provision
            return $cart_item;
        }

        if( isset( $order_item[$this->fieldName] ) ){
            $cart_item_meta[$this->fieldName] = $order_item[$this->fieldName];
        }

		return $cart_item;
    }

    /**
     * Add column to orders
     */

    public function wc_add_order_column( $columns ) {
        $new_columns = [];

        foreach ( $columns as $column_name => $column_info ) {

            $new_columns[ $column_name ] = $column_info;

            if ( 'order_number' === $column_name ) {
                $new_columns[$this->fieldName] = $this->fieldLabel;
            }
        }

        return $new_columns;
    }

	/**
	 * Adds column content to 'Orders' page immediately after 'Total' column.
	 *
	 * @param string[] $column name of column being displayed
	 */
	public function wc_add_order_column_content( $column ) {
		global $post;

        if ($this->fieldName === $column) {
            $order  = wc_get_order( $post->ID );
            $values = $this->field_for_order($this->fieldName, $order);

            foreach ($values as $value) {
                echo $value;
            }
        }
    }

    /**
     * Update meta key for subscription items
     */
    public function wc_order_display_meta_key( $key, $meta, $order_item ) {
        if ($this->fieldName === $key) {
            return $this->fieldLabel;
        }

        return $key;
    }

    /**
     * Add column to subscription page
     */
    public function wc_add_subscription_column( $columns ) {
        $new_columns = [];

        foreach ( $columns as $column_name => $column_info ) {
            $new_columns[ $column_name ] = $column_info;

            if ( 'order_title' === $column_name ) {
                $new_columns[$this->fieldName] = $this->fieldLabel;
            }
        }

        return $new_columns;
    }

	public function wc_add_domain_column_content( $column ) {
		global $post;

		if ( $this->fieldName === $column ) {
			$subscription = new \WC_Subscription($post->ID);
            $values = $this->field_for_order($this->fieldName, $subscription);

            foreach ($values as $value) {
                echo $value;
            }
		}
	}

    public function field_for_order ($fieldName, $order)
    {
        $values = [];

        if ( count( $order->get_items() ) > 0 ) {
            foreach ( $order->get_items() as $item_id => $item ) {
                $values[] = \wc_get_order_item_meta( $item_id, $fieldName, true );
            }
        }

        return $values;
    }
}
