<?php declare(strict_types=1);
namespace DiviManagerPro\Woocommerce;

defined( 'ABSPATH' ) or die( 'Unexpected access' );

use WC_Product_Query;
use WC_Order;
use Wchelper\Wchelper;
use DiviManager\Registry;
use DiviManager\Helper;
use DiviManager\Helper\Option;
use DiviManager\Service\Site as SiteService;
use DiviManager\Service\Profile as ProfileService;
use DiviManagerPro\DiviManagerPro;

class Woocommerce
{
    /**
     * var string
     *
     * The hook prefix for our actions so we can hook into events and stay synchronized with the main plugin
     */
    protected $hookPrefix;

    /**
     * var String
     *
     * The name of the field added to the order for the domain
     */
    protected $domainField = 'website_domain';
    protected $domainLabel = 'Domain';


    /**
     * @var DiviManagerPro
     */
    protected $divimanager;

    /**
     * @var Option
     */
    protected $option;

    public function __construct($hookPrefix, DiviManagerPro $manager)
    {
        $this->hookPrefix = $hookPrefix;
        $this->divimanager = $manager;

        add_action( 'plugins_loaded', [$this, 'plugins_loaded'] );
        add_action( 'init', [$this, 'wp_init'], 13);
        add_action( $this->hookPrefix . '_admin_init', [$this, 'admin_init'], 13);
        add_action( $this->hookPrefix . '_activation', [$this, 'activation'], 13);
        add_action( $this->hookPrefix . '_deactivation', [$this, 'deactivation'], 13);
        add_action( $this->hookPrefix . '_uninstall', [static::class, 'uninstall'], 13);
        add_action( $this->hookPrefix . '_services_init', [$this, 'services_init'], 13);

        //add_action( 'plugins_loaded', [$this, 'update_db_check'] );
    }

    /**
     * DEPRECATED
     */
    /*
    public function deprecated_init()
    {
        $missingDependencies = false;

        if (! $this->isWCSActive()) {
            $missingDependencies = true;
            \add_action( 'admin_notices', function () {
                echo '<div class="error"><p>The DiviManager Woocommerce Extension requires the Woocommerce Subscriptions plugin to be installed and active.</p></div>';
            });

        }

        if (true === $missingDependencies) {
            \deactivate_plugins($mainPluginFile);
            if ( isset( $_GET['activate'] ) ) {
                unset( $_GET['activate'] );
            }

            return false;
        }
    }
     */

    /**
     * function for wordpress init hook
     */
    public function wp_init()
    {
        //add_action( 'wp_loaded', [$this, 'load_wc_products'] );

        if ($this->isWoocommerceEnabled()) {
            /**
             * Add a profile select to products
             * Use the profile value for working with the manager
             *
             * @see https://blog.nexcess.net/custom-fields-woocommerce-products/
             */
            //add_action( 'woocommerce_product_options_pricing', [$this, 'selectProfile'] );
            add_action( 'woocommerce_product_options_general_product_data', [$this, 'selectProfile'] );
            add_action( 'woocommerce_process_product_meta', [$this, 'saveProfile'] );

            // woocommerce automation, after an order is complete
            add_action( 'woocommerce_order_status_completed', [$this, 'orderComplete'], 11, 1);
            //add_action( 'woocommerce_order_status_completed', [$this, 'order_completed'], 10, 1);
            //add_action( 'woocommerce_order_status_processing', [$this, 'order_processing'], 10, 1);

            // woocommerce add domain custom text field
            $field = $this->divimanager->registry->getService(Field::class);
            //$field = new Field($this, $this->domainField, $this->domainLabel);
            add_action( 'woocommerce_before_add_to_cart_button', [$field, 'add_custom_input'], 9 );
            add_filter( 'woocommerce_add_to_cart_validation', [$field, 'add_to_cart_validation'], 10, 3 );
            add_filter( 'woocommerce_add_cart_item_data', [$field, 'add_cart_item_data'], 10, 2 );
            add_filter( 'woocommerce_get_cart_item_from_session', [$field, 'get_cart_item_from_session'], 20, 2 );
            add_action( 'woocommerce_checkout_create_order_line_item', [$field, 'checkout_create_order_line_item'], 10, 4 );
            add_filter( 'woocommerce_get_item_data', [$field, 'get_item_data'], 10, 2 );
            add_filter( 'woocommerce_order_item_product', [$field, 'order_item_product'], 10, 2 );
            add_filter( 'woocommerce_order_again_cart_item_data', [$field, 'order_again_cart_item_data'], 10, 3 );

            // add column to woocommerce orders
            add_filter( 'manage_edit-shop_order_columns', [$field, 'wc_add_order_column'] );
            add_action( 'manage_shop_order_posts_custom_column', [$field, 'wc_add_order_column_content'] );
            add_filter( 'woocommerce_order_item_display_meta_key', [$field, 'wc_order_display_meta_key'], 10, 3 );

            // add column to woocommerce subscriptions
            // @see https://blog.webnersolutions.com/custom-column-in-woocommerce-subscription-list
            add_filter( 'manage_edit-shop_subscription_columns', [$field, 'wc_add_subscription_column'], 20, 1 );
            add_action( 'manage_shop_subscription_posts_custom_column', [$field, 'wc_add_domain_column_content'] );


            if ( is_admin() ) {
                //$admin = new Admin;
                //add_action( 'admin_init', [$admin, 'register_mysettings'] );
                //add_action( 'admin_menu', [$admin, 'admin_menu'] );
            }

            // add wp-cli command
            if ( defined( 'WP_CLI' ) && WP_CLI ) {
                $this->divimanager->registry->setService(Command\Main::class, Command\MainFactory::class);
                $this->divimanager->registry->setService(Command\Renewal::class, Command\RenewalFactory::class);
                $this->divimanager->registry->setService(Command\Cleanup::class, Command\CleanupFactory::class);
                $mainCommand = $this->divimanager->registry->getService(Command\Main::class);
                $renewalCommand = $this->divimanager->registry->getService(Command\Renewal::class);
                $cleanupCommand = $this->divimanager->registry->getService(Command\Cleanup::class);
                //\WP_CLI::add_command( 'manager wc renewal', [$mainCommand, 'renewal'] );
                //\WP_CLI::add_command( 'manager wc renewal', [$mainCommand, 'renewal'] );
                \WP_CLI::add_command( 'manager wc', $mainCommand );
                \WP_CLI::add_command( 'manager wc renewal', $renewalCommand );
                \WP_CLI::add_command( 'manager wc cleanup', $cleanupCommand );
            }
        }
    }

    /**
     * Runs when the admin_init action is called, admin pages are being viewed
     *
     * No need to use an is_admin() check as this is only run before loading admin pages
     */
    public function admin_init()
    {
        $this->divimanager->registry->getService(Admin::class)->init();
		$this->option = $this->divimanager->registry->getService(Option::class); 
    }

    /**
     * Setup our services.
     *
     * Every plugin will be adding their own
     */
    public function services_init(Registry $registry)
    {
        $registry->setService(__CLASS__, $this); 
        $registry->setService(Admin::class, AdminFactory::class);
        $registry->setService(Field::class, FieldFactory::class);
        $registry->setService(Page\WCMenu::class, Page\WCMenuFactory::class);

        // tables
        $registry->setService(Table\Order::class, Table\OrderFactory::class);
    }

    /**
     * install db schema
     *
     * detects the version of the schema
     * if different, runs the script
     */
    protected function install ()
    {
        global $wpdb;

        $installed_ver = get_option( self::VERSION_NAME );

        if ( $installed_ver != self::VERSION ) {
            // set some defaults
            $optionService = $this->divimanager->registry->getService(Option::class);
            $options = $optionService->getOptions();
            $options['woocommerce']['field'] = 'website_domain';
            $options['woocommerce']['label'] = 'Domain';
            $optionService->setOptions($options);

            $table_name = $wpdb->prefix . 'dmpro_wcorder_site'; 

            $charset_collate = $wpdb->get_charset_collate();

            $sql =<<<EOSQL
CREATE TABLE $table_name (
id int NOT NULL AUTO_INCREMENT,
product_id integer NOT NULL,
order_id integer NOT NULL,
time datetime DEFAULT '0000-00-00 00:00:00' NOT NULL,
PRIMARY KEY (id)
) $charset_collate;
EOSQL;

            require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
            dbDelta( $sql );
            update_option( self::VERSION_NAME, self::VERSION, FALSE );
        }
    }

    /**
     * Handle uninstalling and removing any data
     */
    protected function uninstall()
    {
        // currently leave our data in the database
    }

    //public function update_db_check()
    //{
        //if ( get_site_option( self::VERSION_NAME ) != self::VERSION ) {
            //$this->install();
        //}
    //}

    /**
     * On activation of this plugin
     * Also activate the monolog handler
     * @see https://github.com/bradmkjr/monolog-wordpress
     */
    public function activation()
    {
        $this->install();
    }

    /**
     * On deactivation of this plugin
     * Also deactive the monolog handler
     * @see https://github.com/bradmkjr/monolog-wordpress
     */
    public function deactivation()
    {
        // delete the option, set to null?
        //delete_option( self::VERSION_NAME, NULL );
        //update_option( self::VERSION_NAME, NULL, FALSE );
    }

    /**
     * Runs when the plugins_loaded action is called
     * Use this to do db updates
     *
     * Run actions which need to occur every page request, but before wordpress init
     */
    public function plugins_loaded()
    {
    }


    /**
     * Add a select field for the site profile
     *
	 * @see https://blog.nexcess.net/custom-fields-woocommerce-products/
	 * @see https://gist.github.com/jan-koch/5dd8897b974969ed66204e92f6a05ef4
	 */
	public function selectProfile()
    {
        $this->profileService = $this->divimanager->registry->getService(ProfileService::class);
		$profiles = $this->profileService->getProfiles();
		$args = [
			'id' => 'dm_profile', // meta key
			'wrapper_class' => '', // a custom wrapper class if needed
			'desc_tip' => true, // makes your description show up with a "?" symbol and as a tooltip
			'description' => __('If this product is a site plan, select the site profile associated with this product'),
			'label' => __( 'Site Profile' ),
			'options' => [
				'' => 'Not a site',
			]
        ];
		foreach ($profiles as $profile) {
			$args['options'][$profile->getId()] = $profile->getName();
		}

		woocommerce_wp_select( $args ); // FIXME:: does this support a default option for empty value?
	}

    /**
     * Save our profile field data
     *
	 * @see https://gist.github.com/jan-koch/5dd8897b974969ed66204e92f6a05ef4
	 */
	public function saveProfile( $post_id ) {
		// grab the custom SKU from $_POST
		$dm_profile = isset( $_POST[ 'dm_profile' ] ) ? sanitize_text_field( $_POST[ 'dm_profile' ] ) : '';

		// grab the product
		$product = wc_get_product( $post_id );

		// save the custom SKU using WooCommerce built-in functions
		$product->update_meta_data( 'dm_profile', $dm_profile );
		$product->save();
	}

    /**
     * woocommerce order hooks
     */
    /**
     * Triggered when a woocommerce order changes to completed
     *
     * @see https://blog.nexcess.net/custom-fields-woocommerce-products/
     */
    public function orderComplete($order_id)
    {
        $order = new WC_Order($order_id);
        //$this->getLogger()->info(__FUNCTION__ . ' called', ['details' => serialize(['orderId' => $order_id])]);
        if ($this->order_is_renewal($order_id)) {
            //$this->getLogger()->info(__FUNCTION__ . ':Order is a renewal, skipping', ['details' => serialize(['orderId' => $order_id])]);
            return; // order is a renewal, do nothing
        }

        // get our site manager

        // we have the order.  Is it an order for basic/business?
        foreach ($order->get_items() as $item_key => $item_values) {
            $product = $item_values->get_product();
            if ($product instanceof \WC_Product_Subscription_Variation) {
                $product = wc_get_product( $product->get_parent_id() );
            }

            $profile = $this->product_matches_profile($product);
            if (false === $profile) {
                //$this->getLogger()->info('Product is not a matching provision product, skipping', ['details' => serialize(['orderId' => $order_id])]);
                // not a matching product
                continue;
            }

            // url is field in the form?
            $url = $this->getDomainForItem($item_values);
            if (false === $url || empty($url)) {
                //$this->getLogger()->info('Product is missing a url', ['details' => serialize(['order' => $order, 'item' => $item_values, 'meta' => $item_values->get_meta(), 'orderId' => $order_id])]);
                // no url, skip this product
                // FIXME:  This should be an error.  How can a subscription be completed without the domain field being entered?
                continue;
            }

            // tell the manager we want to add a new site
            // the manager will handle the work of connecting to the platform and doing a new install or clone operation
            $this->divimanager->newSite($url, $profile, $order->get_customer_id(), $order_id, 'woocommerce', true, true, true, true);
        }

        //$this->getLogger()->info(__FUNCTION__ . ' done', ['details' => serialize(['orderId' => $order_id])]);
    }

    /**
     * Get all orders with a matching profile
     *
     * @see https://stackoverflow.com/questions/39596477/woocommerce-get-active-subscriptions-in-a-list-between-start-end-date
     */
    public function getProfileOrders()
    {
         $subs = [];
         // get subscriptions
         // check the subscription is a provision
         $subscriptions = \get_posts([
            'numberposts' => -1,
            'post_type'   => 'shop_order', // Subscription post type
            'meta_key' => 'dm_profile',
            'meta_value' => 0,
            'meta_compare' => '>',
            'meta_type' => 'NUMERIC',
            'orderby' => 'post_date', // ordered by date
            'order' => 'ASC',
         ]);
         foreach ($subscriptions as $subscription) {
             $sub = $this->subDataForOrder($subscription);
             if (false === $sub) {
                 continue;
             }

             $subs[] = $sub;
         }

        return array_merge($subs, $this->getProfileSubscriptions());
    }

    /**
     * Get all active subscriptions with a matching profile, and only the subscriptions
     *
     * @see https://stackoverflow.com/questions/39596477/woocommerce-get-active-subscriptions-in-a-list-between-start-end-date
     */
    public function getProfileSubscriptions()
    {
         $subs = [];
         // get subscriptions
         // check the subscription is a provision
         $subscriptions = \get_posts([
            'numberposts' => -1,
            'post_type'   => 'shop_subscription', // Subscription post type
            'post_status' => 'wc-active', // Active subscription
            'orderby' => 'post_date', // ordered by date
            'order' => 'ASC',
            //'date_query' => array( // Start & end date
                //array(
                    //'after'     => $from_date,
                    //'before'    => $to_date,
                    //'inclusive' => true,
                //),
            //),
        ]);
         //echo '<pre>';
         foreach ($subscriptions as $subscription) {
             $sub = $this->subDataForSubscription($subscription);
             if (false === $sub) {
                 continue;
             }

             $subs[] = $sub;
         }

        return $subs;
    }

    public function subDataForOrder($order)
    {
        try {
            $newSub = new \WC_Order($order->ID);
        } catch (\Exception $e) {
            // there weas a problem with this order, ignore it
            return false;
        }
         foreach ($newSub->get_items() as $item_key => $item_values) {
             $product = $item_values->get_product();
             if ($product instanceof \WC_Product_Variation) {
                 $product = wc_get_product( $product->get_parent_id() );
             }

             $profile = $this->product_matches_profile($product);
             if (false === $profile) {
                 continue; // not a provision, skip to next item in the order
             }

            $url = $this->getDomainForItem($item_values);
            if (false === $url || empty($url)) {
                // no url, skip this product
                // FIXME:  This should be an error.  How can a subscription be completed without the domain field being entered?
                continue;
            }

            return [
                'order' => $order->ID,
                'domain' => $url,
                'profile' => $profile,
                'customer' => $newSub->get_customer_id(),
            ];
         }

         return false;
    }

    public function subDataForSubscription($subscription)
    {
         $newSub = new \WC_Subscription($subscription->ID);
         foreach ($newSub->get_items() as $item_key => $item_values) {
             $product = $item_values->get_product();
             if ($product instanceof \WC_Product_Subscription_Variation) {
                 $product = wc_get_product( $product->get_parent_id() );
             }

             $profile = $this->product_matches_profile($product);
             if (false === $profile) {
                 continue; // not a provision, skip to next item in the order
             }

            $url = $this->getDomainForItem($item_values);
            if (false === $url || empty($url)) {
                // no url, skip this product
                // FIXME:  This should be an error.  How can a subscription be completed without the domain field being entered?
                continue;
            }

            return [
                'order' => $subscription->ID,
                'domain' => $url,
                'profile' => $profile,
                'customer' => $newSub->get_customer_id(),
            ];
         }

         return false;
    }

    /**
     * Find any products which need a site to be provisioned/cloned when purchased.
     * Purchases will need to exclude renewal orders
     */
    public function load_wc_products()
    {
        // temp, list products here.
        $query = new WC_Product_Query();
        $query->set( 'tag', ['provision'] );
        $this->products = $query->get_products();
    }

    public function getDomainForItem($item)
    {
        return $item->get_meta($this->domainField);
    }

    /**
     * Is this order a renewal?
     */
    public function order_is_renewal($order_id)
    {
        include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
        if ( \is_plugin_active( 'woocommerce-subscriptions/woocommerce-subscriptions.php' ) ) {
            if ( true === \wcs_order_contains_subscription( $order_id, 'renewal' ) ) {
                return true;
            }
        }
        //$this->getLogger()->info(__FUNCTION__ . ': Not a renewal', ['details' => serialize(['orderId' => $order_id])]);

        return false;
    }

    /**
     * Which provision profile does this product match, if any?
     *
     * @return int|false returns the profile id or false if no profile selected on the product
     */
    public function product_matches_profile(\WC_Product $product)
    {
        if ('variation' === $product->get_type() ||
            'subscription_variation' === $product->get_type()) {
            $product = wc_get_product( $product->get_parent_id() );
        }

        $profile_id = get_post_meta( $product->get_id(), 'dm_profile', true );
        if (null === $profile_id || '' === $profile_id) {
            // no profile set, skip this product
            return false;
        }
        $this->profileService = $this->divimanager->registry->getService(ProfileService::class);
        return $this->profileService->getProfile($profile_id);
    }

    /**
     * Wrap checks for an active plugin
     */
    public function isWCSActive()
    {
		// @see https://docs.woocommerce.com/document/create-a-plugin/
		return in_array( 'woocommerce/woocommerce.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) );
    }

    /**
     * Whether the Woocommerce functionality is enabled
     *
     * @return bool
     */
    public function isWoocommerceEnabled()
    {
        if ($this->isWCSActive()) {
            $this->option = $this->divimanager->registry->getService(Option::class);
            return true == $this->option->getOption('woocommerce_enabled');
        }

        return false;
    }
}
