<?php

namespace ServerPilotAPI;

use GuzzleHttp\Client;

class ServerPilotAPI
{
    /**
     * @var String Base site url for api
     */
    protected $siteUrl;

    /**
     * @var String credential for account
     */
    protected $credential;

    /**
     * @var String authentication client id
     */
    protected $clientID;

    /**
     * @var String authentication api key
     */
    protected $apiKey;

    /**
     * @var Client our GuzzleHttp/Client instance
     */
    protected $guzzle;

    /**
     * values for new sites
     */
    protected $adminUser;
    protected $adminEmail;
    protected $phpRuntime;

    /**
     * Constructor for creating a new ServerPilot API connection instance
     */
    public function __construct($siteUrl, $clientID, $apiKey)
    {
        $this->siteUrl = $siteUrl;
        $this->clientID = $clientID;
        $this->apiKey = $apiKey;
        $this->guzzle = null;

        $this->start();
    }

    protected function getServerRequest()
    {
        $req = new Request\Server($this->guzzle, $this->siteUrl);
        $req->setClientID($this->clientID);
        $req->setApiKey($this->apiKey);

        return $req;
    }

    protected function getActionRequest()
    {
        $site = new Request\Action($this->guzzle, $this->siteUrl);
        $site->setClientID($this->clientID);
        $site->setApiKey($this->apiKey);
        return $site;
    }

    protected function getSiteRequest()
    {
        $site = new Request\Site($this->guzzle, $this->siteUrl);
        $site->setClientID($this->clientID);
        $site->setApiKey($this->apiKey);
        $site->setAdminUser($this->adminUser);
        $site->setAdminEmail($this->adminEmail);
        $site->setPhpRuntime($this->phpRuntime);

        return $site;
    }

    protected function getSshkeyRequest()
    {
        $site = new Request\Sshkey($this->guzzle, $this->siteUrl);
        $site->setClientID($this->clientID);
        $site->setApiKey($this->apiKey);

        return $site;
    }

    protected function getSysuserRequest()
    {
        $site = new Request\Sysuser($this->guzzle, $this->siteUrl);
        $site->setClientID($this->clientID);
        $site->setApiKey($this->apiKey);

        return $site;
    }

    protected function getWorker()
    {
        $worker = new Worker();
        return $worker;
    }

    public function start()
    {
        $this->guzzle = new Client(['base_uri' => $this->siteUrl]);
    }

    /**
     * Close client
     */
    public function close()
    {
        // serverpilotapi does not require separate login/logout operations
    }

    /**
     * Login not supported in the serverpilot api
     */
    public function login()
    {
    }

    /**
     * Logout not supported in the serverpilot api
     */
    public function logout()
    {
    }

    /**
     * Get servers
     */
    public function servers()
    {
        return $this->getServerRequest()->getList();
    }

    /**
     * Get my sites
     */
    public function sites()
    {
        $site = $this->getSiteRequest();
        return $site->getList();
    }

    /**
     * Get site details
     */
    public function site($id)
    {
        $site = $this->getSiteRequest();
        return $site->get($id);
    }

    /**
     * Create a new site
     */
    public function createSite($name, $user, $domain, $runtime)
    {
        $site = $this->getSiteRequest();
        return $site->create($name, $user, $domain, $runtime);
    }

    /**
     * Enable autossl on a site
     */
    public function autosslSite($site)
    {
        $request = $this->getSiteRequest();
        return $request->autossl($site);
    }

    /**
     * Enable force-ssl on a site
     */
    public function forcesslSite($site)
    {
        $request = $this->getSiteRequest();
        return $request->forcessl($site);
    }

    /**
     * Add domain to site
     */
    public function addDomainToSite($domain, $site)
    {
        $request = $this->getSiteRequest();
        $data = $request->get($site);
        $domains = $data['domains'];
        foreach ($domains as $existing) {
            if ($existing == $domain) {
                // domain already on site
                return;
            }
        }
        $domains[] = $domain;
        return $request->updateDomains($site, $domains);
    }

    /**
     * Remove domain from site
     */
    public function removeDomainFromSite($domain, $site)
    {
        $request = $this->getSiteRequest();
        $data = $request->get($site);
        $domains = $data['domains'];
        $found = false;
        foreach ($domains as $key => $existing) {
            if ($existing == $domain) {
                // found it
                $found = true;
                unset($domains[$key]);
            }
        }
        if ($found) {
            return $request->updateDomains($site, $domains);
        } else {
            return;
        }
    }

    /**
     * Get path information
     */
    public function paths($siteID)
    {
        $site = $this->getSiteRequest()->get($siteID);
        $user = $this->getSysuserRequest()->get($site['sysuserid']);
        return [$user->name, $site['name']];
    }

    /**
     * Clone site
     */
    public function cloneSite($sourceID, $destID)
    {
        throw new \RuntimeException('do not use');
/*
        // get the user and path for the source id
        $src = $this->getSiteRequest()->get($sourceID);
        var_dump($src);
        $srcUser = $this->getSysuserRequest()->get($src->sysuserid);
        // get the user and path for the destination
        $dest = $this->getSiteRequest()->get($destID);
        $destUser = $this->getSysuserRequest()->get($dest->sysuserid);
        //throw new \RuntimeException('not yet supported');

        $messages = $this->getWorker()->migrate($srcUser->name, $src->name, $destUser->name, $dest->name);
        $site = $this->getSiteRequest();
        return $site->create($id, $domain);
 */
    }

    /**
     * List ssh keys
     */
    public function sshkeys()
    {
        $req = $this->getSshkeyRequest();
        return $req->getList();
    }

    public function sysusers()
    {
        $req = $this->getSysuserRequest();
        return $req->getList();
    }

    /**
     * Add sshkey to sysuser
     */
    public function addKeyToUser($key, $user)
    {
        $req = $this->getSysuserRequest();
        return $req->addKeyToUser($key, $user);
    }

    /**
     * Create a new user
     */
    public function createUser($server, $name, $password, $sshKey)
    {
        return $this->getSysuserRequest()->create($server, $name, $password, $sshKey);
    }

    /**
     * Check status of an action
     */
    public function checkAction($actionid)
    {
        return $this->getActionRequest()->get($actionid);
    }

    /**
     * is the action still open?
     */
    public function actionOpen($actionid)
    {
        return Request\Action::OPEN === $this->getActionRequest()->get($actionid);
    }

    /**
     * is the action successful?
     */
    public function actionSuccess($actionid)
    {
        return Request\Action::SUCCESS === $this->getActionRequest()->get($actionid);
    }

    public function setAdminUser($adminUser)
    {
        $this->adminUser = $adminUser;
    }
    public function setAdminEmail($adminEmail)
    {
        $this->adminEmail = $adminEmail;
    }
    public function setPhpRuntime($phpRuntime)
    {
        $this->phpRuntime = $phpRuntime;
    }

    public function getGuzzle()
    {
        return $this->guzzle;
    }
}
